# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/arrow/v1/arrow.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1a\x63halk/arrow/v1/arrow.proto\x12\x0e\x63halk.arrow.v1"\xb8\x01\n\x06Schema\x12/\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\x07\x63olumns\x12@\n\x08metadata\x18\x02 \x03(\x0b\x32$.chalk.arrow.v1.Schema.MetadataEntryR\x08metadata\x1a;\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01"\xa2\x02\n\x05\x46ield\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x38\n\narrow_type\x18\x02 \x01(\x0b\x32\x19.chalk.arrow.v1.ArrowTypeR\tarrowType\x12\x1a\n\x08nullable\x18\x03 \x01(\x08R\x08nullable\x12\x31\n\x08\x63hildren\x18\x04 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\x08\x63hildren\x12?\n\x08metadata\x18\x05 \x03(\x0b\x32#.chalk.arrow.v1.Field.MetadataEntryR\x08metadata\x1a;\n\rMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01")\n\x0f\x46ixedSizeBinary\x12\x16\n\x06length\x18\x01 \x01(\x05R\x06length"^\n\tTimestamp\x12\x35\n\ttime_unit\x18\x01 \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitR\x08timeUnit\x12\x1a\n\x08timezone\x18\x02 \x01(\tR\x08timezone"=\n\x07\x44\x65\x63imal\x12\x1c\n\tprecision\x18\x01 \x01(\x05R\tprecision\x12\x14\n\x05scale\x18\x02 \x01(\x05R\x05scale"<\n\x04List\x12\x34\n\nfield_type\x18\x01 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\tfieldType"b\n\rFixedSizeList\x12\x34\n\nfield_type\x18\x01 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\tfieldType\x12\x1b\n\tlist_size\x18\x02 \x01(\x05R\x08listSize"G\n\x06Struct\x12=\n\x0fsub_field_types\x18\x01 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\rsubFieldTypes"\x90\x01\n\x03Map\x12\x32\n\tkey_field\x18\x01 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\x08keyField\x12\x34\n\nitem_field\x18\x02 \x01(\x0b\x32\x15.chalk.arrow.v1.FieldR\titemField\x12\x1f\n\x0bkeys_sorted\x18\x03 \x01(\x08R\nkeysSorted"`\n\x0fScalarListValue\x12\x1d\n\narrow_data\x18\x01 \x01(\x0cR\tarrowData\x12.\n\x06schema\x18\x02 \x01(\x0b\x32\x16.chalk.arrow.v1.SchemaR\x06schema"\x8a\x01\n\x11ScalarTime32Value\x12\x30\n\x13time32_second_value\x18\x01 \x01(\x05H\x00R\x11time32SecondValue\x12:\n\x18time32_millisecond_value\x18\x02 \x01(\x05H\x00R\x16time32MillisecondValueB\x07\n\x05value"\x92\x01\n\x11ScalarTime64Value\x12:\n\x18time64_microsecond_value\x18\x01 \x01(\x03H\x00R\x16time64MicrosecondValue\x12\x38\n\x17time64_nanosecond_value\x18\x02 \x01(\x03H\x00R\x15time64NanosecondValueB\x07\n\x05value"\x8f\x02\n\x14ScalarTimestampValue\x12\x36\n\x16time_microsecond_value\x18\x01 \x01(\x03H\x00R\x14timeMicrosecondValue\x12\x34\n\x15time_nanosecond_value\x18\x02 \x01(\x03H\x00R\x13timeNanosecondValue\x12,\n\x11time_second_value\x18\x03 \x01(\x03H\x00R\x0ftimeSecondValue\x12\x36\n\x16time_millisecond_value\x18\x04 \x01(\x03H\x00R\x14timeMillisecondValue\x12\x1a\n\x08timezone\x18\x05 \x01(\tR\x08timezoneB\x07\n\x05value"|\n\x0bStructValue\x12>\n\x0c\x66ield_values\x18\x02 \x03(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x0b\x66ieldValues\x12-\n\x06\x66ields\x18\x03 \x03(\x0b\x32\x15.chalk.arrow.v1.FieldR\x06\x66ields"G\n\x15ScalarFixedSizeBinary\x12\x16\n\x06values\x18\x01 \x01(\x0cR\x06values\x12\x16\n\x06length\x18\x02 \x01(\x05R\x06length"X\n\x0c\x44\x65\x63imalValue\x12\x14\n\x05value\x18\x01 \x01(\x0cR\x05value\x12\x1c\n\tprecision\x18\x02 \x01(\x05R\tprecision\x12\x14\n\x05scale\x18\x03 \x01(\x05R\x05scale"]\n\tExtension\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12<\n\x0cstorage_type\x18\x02 \x01(\x0b\x32\x19.chalk.arrow.v1.ArrowTypeR\x0bstorageType"\x94\x01\n\x0e\x45xtensionValue\x12@\n\x0e\x65xtension_type\x18\x01 \x01(\x0b\x32\x19.chalk.arrow.v1.ExtensionR\rextensionType\x12@\n\rstorage_value\x18\x02 \x01(\x0b\x32\x1b.chalk.arrow.v1.ScalarValueR\x0cstorageValue"\xef\x0e\n\x0bScalarValue\x12:\n\nnull_value\x18\x01 \x01(\x0b\x32\x19.chalk.arrow.v1.ArrowTypeH\x00R\tnullValue\x12\x1f\n\nbool_value\x18\x02 \x01(\x08H\x00R\tboolValue\x12%\n\rfloat64_value\x18\x03 \x01(\x01H\x00R\x0c\x66loat64Value\x12!\n\x0bint64_value\x18\x04 \x01(\x03H\x00R\nint64Value\x12*\n\x10large_utf8_value\x18\x05 \x01(\tH\x00R\x0elargeUtf8Value\x12O\n\x0ftimestamp_value\x18\x06 \x01(\x0b\x32$.chalk.arrow.v1.ScalarTimestampValueH\x00R\x0etimestampValue\x12$\n\rdate_64_value\x18\x07 \x01(\x03H\x00R\x0b\x64\x61te64Value\x12@\n\x0cstruct_value\x18\x08 \x01(\x0b\x32\x1b.chalk.arrow.v1.StructValueH\x00R\x0bstructValue\x12K\n\x10large_list_value\x18\t \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\x0elargeListValue\x12\x46\n\x0ctime64_value\x18\n \x01(\x0b\x32!.chalk.arrow.v1.ScalarTime64ValueH\x00R\x0btime64Value\x12\x34\n\x15\x64uration_second_value\x18\x10 \x01(\x03H\x00R\x13\x64urationSecondValue\x12>\n\x1a\x64uration_millisecond_value\x18\x11 \x01(\x03H\x00R\x18\x64urationMillisecondValue\x12>\n\x1a\x64uration_microsecond_value\x18\x12 \x01(\x03H\x00R\x18\x64urationMicrosecondValue\x12<\n\x19\x64uration_nanosecond_value\x18\x13 \x01(\x03H\x00R\x17\x64urationNanosecondValue\x12\x1f\n\nutf8_value\x18\x14 \x01(\tH\x00R\tutf8Value\x12\x1f\n\nint8_value\x18\x15 \x01(\x05H\x00R\tint8Value\x12!\n\x0bint16_value\x18\x16 \x01(\x05H\x00R\nint16Value\x12!\n\x0bint32_value\x18\x17 \x01(\x05H\x00R\nint32Value\x12!\n\x0buint8_value\x18\x18 \x01(\rH\x00R\nuint8Value\x12#\n\x0cuint16_value\x18\x19 \x01(\rH\x00R\x0buint16Value\x12#\n\x0cuint32_value\x18\x1a \x01(\rH\x00R\x0buint32Value\x12#\n\x0cuint64_value\x18\x1b \x01(\x04H\x00R\x0buint64Value\x12%\n\rfloat16_value\x18\x1c \x01(\x02H\x00R\x0c\x66loat16Value\x12%\n\rfloat32_value\x18\x1d \x01(\x02H\x00R\x0c\x66loat32Value\x12$\n\rdate_32_value\x18\x1e \x01(\x05H\x00R\x0b\x64\x61te32Value\x12\x46\n\x0ctime32_value\x18\x1f \x01(\x0b\x32!.chalk.arrow.v1.ScalarTime32ValueH\x00R\x0btime32Value\x12@\n\nlist_value\x18  \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\tlistValue\x12T\n\x15\x66ixed_size_list_value\x18! \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\x12\x66ixedSizeListValue\x12>\n\tmap_value\x18" \x01(\x0b\x32\x1f.chalk.arrow.v1.ScalarListValueH\x00R\x08mapValue\x12#\n\x0c\x62inary_value\x18# \x01(\x0cH\x00R\x0b\x62inaryValue\x12.\n\x12large_binary_value\x18$ \x01(\x0cH\x00R\x10largeBinaryValue\x12^\n\x17\x66ixed_size_binary_value\x18% \x01(\x0b\x32%.chalk.arrow.v1.ScalarFixedSizeBinaryH\x00R\x14\x66ixedSizeBinaryValue\x12I\n\x10\x64\x65\x63imal128_value\x18& \x01(\x0b\x32\x1c.chalk.arrow.v1.DecimalValueH\x00R\x0f\x64\x65\x63imal128Value\x12I\n\x10\x64\x65\x63imal256_value\x18\' \x01(\x0b\x32\x1c.chalk.arrow.v1.DecimalValueH\x00R\x0f\x64\x65\x63imal256Value\x12I\n\x0f\x65xtension_value\x18( \x01(\x0b\x32\x1e.chalk.arrow.v1.ExtensionValueH\x00R\x0e\x65xtensionValueB\x07\n\x05value"\x89\x0e\n\tArrowType\x12\x32\n\x04none\x18\x01 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04none\x12\x32\n\x04\x62ool\x18\x02 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04\x62ool\x12\x38\n\x07\x66loat64\x18\x03 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x07\x66loat64\x12\x34\n\x05int64\x18\x04 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05int64\x12=\n\nlarge_utf8\x18\x05 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\tlargeUtf8\x12\x39\n\ttimestamp\x18\x06 \x01(\x0b\x32\x19.chalk.arrow.v1.TimestampH\x00R\ttimestamp\x12\x36\n\x06\x64\x61te64\x18\x07 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06\x64\x61te64\x12\x30\n\x06struct\x18\x08 \x01(\x0b\x32\x16.chalk.arrow.v1.StructH\x00R\x06struct\x12\x35\n\nlarge_list\x18\t \x01(\x0b\x32\x14.chalk.arrow.v1.ListH\x00R\tlargeList\x12\x32\n\x06time64\x18\n \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitH\x00R\x06time64\x12\x36\n\x08\x64uration\x18\x10 \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitH\x00R\x08\x64uration\x12\x32\n\x04utf8\x18\x11 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04utf8\x12\x32\n\x04int8\x18\x12 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x04int8\x12\x34\n\x05int16\x18\x13 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05int16\x12\x34\n\x05int32\x18\x14 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05int32\x12\x34\n\x05uint8\x18\x15 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x05uint8\x12\x36\n\x06uint16\x18\x16 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06uint16\x12\x36\n\x06uint32\x18\x17 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06uint32\x12\x36\n\x06uint64\x18\x18 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06uint64\x12\x38\n\x07\x66loat16\x18\x19 \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x07\x66loat16\x12\x38\n\x07\x66loat32\x18\x1a \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x07\x66loat32\x12\x36\n\x06\x64\x61te32\x18\x1b \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06\x64\x61te32\x12\x32\n\x06time32\x18\x1c \x01(\x0e\x32\x18.chalk.arrow.v1.TimeUnitH\x00R\x06time32\x12*\n\x04list\x18\x1d \x01(\x0b\x32\x14.chalk.arrow.v1.ListH\x00R\x04list\x12G\n\x0f\x66ixed_size_list\x18\x1e \x01(\x0b\x32\x1d.chalk.arrow.v1.FixedSizeListH\x00R\rfixedSizeList\x12\x36\n\x06\x62inary\x18\x1f \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x06\x62inary\x12\x41\n\x0clarge_binary\x18  \x01(\x0b\x32\x1c.chalk.arrow.v1.EmptyMessageH\x00R\x0blargeBinary\x12,\n\x11\x66ixed_size_binary\x18! \x01(\x05H\x00R\x0f\x66ixedSizeBinary\x12:\n\x0b\x64\x65\x63imal_128\x18" \x01(\x0b\x32\x17.chalk.arrow.v1.DecimalH\x00R\ndecimal128\x12:\n\x0b\x64\x65\x63imal_256\x18# \x01(\x0b\x32\x17.chalk.arrow.v1.DecimalH\x00R\ndecimal256\x12\'\n\x03map\x18$ \x01(\x0b\x32\x13.chalk.arrow.v1.MapH\x00R\x03map\x12\x39\n\textension\x18% \x01(\x0b\x32\x19.chalk.arrow.v1.ExtensionH\x00R\textensionB\x11\n\x0f\x61rrow_type_enum"\x0e\n\x0c\x45mptyMessage*\x8b\x01\n\x08TimeUnit\x12\x19\n\x15TIME_UNIT_UNSPECIFIED\x10\x00\x12\x14\n\x10TIME_UNIT_SECOND\x10\x01\x12\x19\n\x15TIME_UNIT_MILLISECOND\x10\x02\x12\x19\n\x15TIME_UNIT_MICROSECOND\x10\x03\x12\x18\n\x14TIME_UNIT_NANOSECOND\x10\x04*\x9a\x01\n\x12\x46\x65\x61therCompression\x12#\n\x1f\x46\x45\x41THER_COMPRESSION_UNSPECIFIED\x10\x00\x12$\n FEATHER_COMPRESSION_UNCOMPRESSED\x10\x01\x12\x1b\n\x17\x46\x45\x41THER_COMPRESSION_LZ4\x10\x02\x12\x1c\n\x18\x46\x45\x41THER_COMPRESSION_ZSTD\x10\x03\x42z\n\x12\x63om.chalk.arrow.v1B\nArrowProtoP\x01\xa2\x02\x03\x43\x41X\xaa\x02\x0e\x43halk.Arrow.V1\xca\x02\x0e\x43halk\\Arrow\\V1\xe2\x02\x1a\x43halk\\Arrow\\V1\\GPBMetadata\xea\x02\x10\x43halk::Arrow::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.arrow.v1.arrow_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\022com.chalk.arrow.v1B\nArrowProtoP\001\242\002\003CAX\252\002\016Chalk.Arrow.V1\312\002\016Chalk\\Arrow\\V1\342\002\032Chalk\\Arrow\\V1\\GPBMetadata\352\002\020Chalk::Arrow::V1"
    _globals["_SCHEMA_METADATAENTRY"]._options = None
    _globals["_SCHEMA_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_FIELD_METADATAENTRY"]._options = None
    _globals["_FIELD_METADATAENTRY"]._serialized_options = b"8\001"
    _globals["_TIMEUNIT"]._serialized_start = 6034
    _globals["_TIMEUNIT"]._serialized_end = 6173
    _globals["_FEATHERCOMPRESSION"]._serialized_start = 6176
    _globals["_FEATHERCOMPRESSION"]._serialized_end = 6330
    _globals["_SCHEMA"]._serialized_start = 47
    _globals["_SCHEMA"]._serialized_end = 231
    _globals["_SCHEMA_METADATAENTRY"]._serialized_start = 172
    _globals["_SCHEMA_METADATAENTRY"]._serialized_end = 231
    _globals["_FIELD"]._serialized_start = 234
    _globals["_FIELD"]._serialized_end = 524
    _globals["_FIELD_METADATAENTRY"]._serialized_start = 172
    _globals["_FIELD_METADATAENTRY"]._serialized_end = 231
    _globals["_FIXEDSIZEBINARY"]._serialized_start = 526
    _globals["_FIXEDSIZEBINARY"]._serialized_end = 567
    _globals["_TIMESTAMP"]._serialized_start = 569
    _globals["_TIMESTAMP"]._serialized_end = 663
    _globals["_DECIMAL"]._serialized_start = 665
    _globals["_DECIMAL"]._serialized_end = 726
    _globals["_LIST"]._serialized_start = 728
    _globals["_LIST"]._serialized_end = 788
    _globals["_FIXEDSIZELIST"]._serialized_start = 790
    _globals["_FIXEDSIZELIST"]._serialized_end = 888
    _globals["_STRUCT"]._serialized_start = 890
    _globals["_STRUCT"]._serialized_end = 961
    _globals["_MAP"]._serialized_start = 964
    _globals["_MAP"]._serialized_end = 1108
    _globals["_SCALARLISTVALUE"]._serialized_start = 1110
    _globals["_SCALARLISTVALUE"]._serialized_end = 1206
    _globals["_SCALARTIME32VALUE"]._serialized_start = 1209
    _globals["_SCALARTIME32VALUE"]._serialized_end = 1347
    _globals["_SCALARTIME64VALUE"]._serialized_start = 1350
    _globals["_SCALARTIME64VALUE"]._serialized_end = 1496
    _globals["_SCALARTIMESTAMPVALUE"]._serialized_start = 1499
    _globals["_SCALARTIMESTAMPVALUE"]._serialized_end = 1770
    _globals["_STRUCTVALUE"]._serialized_start = 1772
    _globals["_STRUCTVALUE"]._serialized_end = 1896
    _globals["_SCALARFIXEDSIZEBINARY"]._serialized_start = 1898
    _globals["_SCALARFIXEDSIZEBINARY"]._serialized_end = 1969
    _globals["_DECIMALVALUE"]._serialized_start = 1971
    _globals["_DECIMALVALUE"]._serialized_end = 2059
    _globals["_EXTENSION"]._serialized_start = 2061
    _globals["_EXTENSION"]._serialized_end = 2154
    _globals["_EXTENSIONVALUE"]._serialized_start = 2157
    _globals["_EXTENSIONVALUE"]._serialized_end = 2305
    _globals["_SCALARVALUE"]._serialized_start = 2308
    _globals["_SCALARVALUE"]._serialized_end = 4211
    _globals["_ARROWTYPE"]._serialized_start = 4214
    _globals["_ARROWTYPE"]._serialized_end = 6015
    _globals["_EMPTYMESSAGE"]._serialized_start = 6017
    _globals["_EMPTYMESSAGE"]._serialized_end = 6031
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import ast
import re
import sys
import types
import typing
from datetime import timedelta
from typing import TYPE_CHECKING, Any, Dict, FrozenSet, List, Optional, Set, Tuple, Type, TypeVar, Union, cast

from typing_extensions import Annotated, get_args, get_origin

from chalk._lsp.error_builder import LSPErrorBuilder
from chalk.features._encoding.http import HttpResponse
from chalk.utils.cached_type_hints import cached_get_type_hints
from chalk.utils.collections import is_optional
from chalk.utils.import_utils import (
    gather_all_imports_and_local_classes,
    get_detailed_type_hint_errors,
    import_only_type_checking_imports,
)
from chalk.utils.json import JSON
from chalk.utils.log_with_context import get_logger

if TYPE_CHECKING:
    from google.protobuf.message import Message as ProtobufMessage

    from chalk import Document, Windowed
    from chalk.features import DataFrame, Feature, Features, Vector
    from chalk.streams import Windowed

_logger = get_logger(__name__)

T = TypeVar("T")
U = TypeVar("U")
JsonValue = TypeVar("JsonValue")


class ParsedAnnotation:
    __slots__ = (
        "_features_cls",
        "_attribute_name",
        "_is_nullable",
        "_is_feature_time",
        "_is_primary",
        "_is_document",
        "_underlying",
        "_unparsed_underlying",
        "_parsed_annotation",
        "_as_proto",
        "_as_document",
        "_as_features_cls",
        "_as_dataframe",
        "_as_vector",
        "_as_feature",
    )

    def __init__(
        self,
        features_cls: Optional[Type[Features]] = None,
        attribute_name: Optional[str] = None,
        *,
        underlying: Optional[Union[type, Annotated, Windowed]] = None,
    ) -> None:
        super().__init__()
        # Either pass in the underlying -- if it is already parsed -- or pass in the feature cls and attribute name
        self._features_cls = features_cls
        self._attribute_name = attribute_name
        self._is_nullable = False
        self._is_feature_time = False
        self._is_primary = False
        self._is_document = False
        self._underlying: Optional[Union[type, Feature]] = None
        self._unparsed_underlying = underlying
        self._parsed_annotation: Optional[type] = None
        if underlying is not None:
            if features_cls is not None and attribute_name is not None:
                raise ValueError("If specifying the underlying, do not specify (features_cls, attribute_name)")
        elif features_cls is None or attribute_name is None:
            raise ValueError(
                "If not specifying the underlying, then both the (features_cls, attribute_name) must be provided"
            )
        # Store the class and attribute name to later use typing.get_type_hints to
        # resolve any forward references in the type annotations
        # Resolution happens lazily -- after everything is imported -- to avoid circular imports

        # Caching some properties for faster lookup
        self._as_proto: Optional[Type[ProtobufMessage]] = None
        self._as_document: Optional[Type[Document]] = None
        self._as_features_cls: Optional[Type[Features]] = None
        self._as_dataframe: Optional[Type[DataFrame]] = None
        self._as_vector: Optional[Type[Vector]] = None
        self._as_feature: Optional[Feature] = None

    @property
    def parsed_annotation(self) -> type:
        """The parsed type annotation. It will be parsed if needed.

        Unlike `.underlying`, parsed annotation contains any container or optional types, such as
        list, dataframe, or Optional.
        """
        if self._parsed_annotation is None:
            self._parse_annotation()
        assert self._parsed_annotation is not None
        return self._parsed_annotation

    def __str__(self):
        if self._features_cls is not None and self._attribute_name is not None:
            typ = self._features_cls.__annotations__[self._attribute_name]
        else:
            typ = self._parsed_annotation or self._underlying or self._unparsed_underlying
        if sys.version_info >= (3, 9) and get_origin(typ) == typing.Annotated:
            typ = typ.__origin__  # pyright: ignore
        if isinstance(typ, type):
            typ = typ.__name__
        return str(typ)

    def _get_globals_for_forward_references(self) -> Dict[str, Any]:
        """
        If we're loading pickles from a notebook onto the branch server OR if we are loading forward references
        from separate files, we need special handling for forward references.
        """

        import typing

        import chalk
        from chalk.features.feature_set import CURRENT_FEATURE_REGISTRY

        registry = CURRENT_FEATURE_REGISTRY.get()
        feature_classes = {feature_set.__name__: feature_set for feature_set in registry.get_feature_sets().values()}

        if not getattr(self._features_cls, "__chalk_is_loaded_from_notebook__", False):
            return feature_classes

        return {
            # Support "typing.Optional[...]", "Optional[...]", etc."
            "typing": typing,
            **typing.__dict__,
            # Support "chalk.DataFrame[...]", "DataFrame[...]"
            "chalk": chalk,
            **chalk.__dict__,
            # Support forward references for cyclic has-one/has-many's
            **feature_classes,
        }

    def _parse_annotation(self):
        if self._unparsed_underlying is None:
            assert self._attribute_name is not None
            assert self._features_cls is not None
            try:
                # First, try to get the type hints without doing anything fancy
                # Everything should already be imported properly
                hints = cached_get_type_hints(self._features_cls, include_extras=True)
            except:
                # If there was an issue with that, then we'll fallback to trying to augment the globals, automatically injecting
                # feature sets and chalkpy imports under their respective names
                module = sys.modules.get(self._features_cls.__module__, None)
                existing_globalns = {}
                if module is not None:
                    existing_globalns.update(getattr(module, "__dict__", {}))
                existing_globalns.update(self._get_globals_for_forward_references())
                try:
                    hints = cached_get_type_hints(
                        self._features_cls,
                        include_extras=True,
                        globalns=existing_globalns if len(existing_globalns) > 0 else None,
                    )
                except:
                    # If there was an issue with THAT, we'll try to import files specified by
                    # TYPE_CHECKING only imports, and augment the globals again
                    tree: ast.Module | None = None
                    if self._features_cls.__chalk_source_info__.filename is not None:
                        tree = import_only_type_checking_imports(self._features_cls.__chalk_source_info__.filename)
                    existing_globalns.update(self._get_globals_for_forward_references())
                    try:
                        hints = cached_get_type_hints(
                            self._features_cls,
                            include_extras=True,
                            globalns=existing_globalns if len(existing_globalns) > 0 else None,
                        )
                    except:
                        # Last shot, in a notebook context, we may not have access to annotation types
                        import datetime as dt
                        from typing import Optional

                        from chalk import Windowed
                        from chalk.features import DataFrame, FeatureTime, Primary

                        existing_globalns.update(
                            {
                                "Optional": Optional,
                                "DataFrame": DataFrame,
                                "Primary": Primary,
                                "FeatureTime": FeatureTime,
                                "dt": dt,
                                "Windowed": Windowed,
                            }
                        )
                        try:
                            hints = cached_get_type_hints(
                                self._features_cls,
                                include_extras=True,
                                globalns=existing_globalns if len(existing_globalns) > 0 else None,
                            )
                        except Exception as e:
                            # At this point, we've failed. Let's try to get a better error message at the very least.
                            name_error_pattern = re.compile(r"name '([^']*)' is not defined")
                            attribute_errors = get_detailed_type_hint_errors(
                                self._features_cls, True, existing_globalns if len(existing_globalns) > 0 else None
                            )
                            if isinstance(e, NameError) and tree is not None:
                                feature_namespace = self._features_cls.namespace
                                for attribute_name, error in attribute_errors.items():
                                    match = name_error_pattern.search(str(error))
                                    if match is not None:
                                        missing_import_string = match.group(1)
                                        (
                                            regular_from_imports,
                                            type_checking_imports,
                                            local_classes,
                                        ) = gather_all_imports_and_local_classes(tree)
                                        if missing_import_string not in regular_from_imports + type_checking_imports:
                                            if missing_import_string in local_classes:
                                                # Feature class is defined somewhere else in the same file, so let's skip
                                                continue
                                            builder = self._features_cls and self._features_cls.__chalk_error_builder__
                                            assert builder is not None
                                            builder.add_diagnostic(
                                                message=(
                                                    f"Feature class '{feature_namespace}.{attribute_name}' has type annotation referencing '{missing_import_string}', "
                                                    + f"which is undefined. Please import this using IS_TYPE_CHECKING. "
                                                ),
                                                label="invalid annotation",
                                                code="79",
                                                range=builder.annotation_range(attribute_name),
                                            )
                            raise TypeError(
                                f"Could not get type hints of feature class '{self._features_cls}' from filename {self._features_cls.__chalk_source_info__.filename}: {str(e)}"
                            ) from e

            parsed_annotation = hints[self._attribute_name]
        else:
            parsed_annotation = self._unparsed_underlying

        try:
            self._parse_type(parsed_annotation)
        except Exception as e:
            if (
                # This is our catch-all, which we only want to run if someone else didn't already report an error.
                not LSPErrorBuilder.has_errors()
                and self._features_cls is not None
                and self._features_cls.__chalk_error_builder__
                is not None  # pyright: ignore[reportUnnecessaryComparison]
            ):
                try:
                    self._type_error(message=e.args[0], code="79")
                except:
                    pass
            raise e

    def _type_error(
        self,
        message: str,
        code: str,
        label: str = "invalid annotation",
        code_href: Optional[str] = "https://docs.chalk.ai/docs/feature-types",
    ):
        builder = self._features_cls and self._features_cls.__chalk_error_builder__
        if builder is not None:
            builder.add_diagnostic(
                message=message,
                label=label,
                code=code,
                code_href=code_href,
                range=(builder.annotation_range(self._attribute_name) or builder.property_range(self._attribute_name))
                if self._attribute_name
                else builder.class_definition_range(),
            )
        raise TypeError(message)

    def _parse_type(self, annotation: Union[type, Windowed, Annotated]):
        from chalk.features._vector import Vector
        from chalk.features.dataframe import DataFrame
        from chalk.features.feature_field import Feature
        from chalk.features.feature_set import Features
        from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
        from chalk.streams import Windowed

        try:
            from google.protobuf.message import Message as ProtobufMessage
        except ImportError:
            ProtobufMessage = None

        # assert self._parsed_annotation is None, "The annotation was already parsed"
        if isinstance(annotation, Windowed):
            # If it's windowed, then unwrap it immediately, because Windowed annotations are really just a proxy to the underlying type
            annotation = annotation.kind
        self._parsed_annotation = cast(type, annotation)
        self._is_nullable = False
        self._is_primary = False
        self._is_feature_time = False
        if self._features_cls is not None and self._attribute_name is not None:
            # Return a more helpful error message, since we have context
            error_ctx = f"{self._features_cls.__name__}.{self._attribute_name}"
        else:
            error_ctx = ""
        origin = get_origin(annotation)

        if annotation is JSON:
            self._is_nullable = True
        elif origin in (
            Union,
            getattr(types, "UnionType", Union),
        ):  # using getattr as UnionType was introduced in python 3.10
            args = get_args(annotation)
            # If it's a union, then the only supported union is for nullable features. Validate this
            if len(args) != 2 or (None not in args and type(None) not in args):
                self._type_error(
                    f"Invalid annotation for feature {error_ctx}: Unions with non-None types are not allowed.",
                    code="71",
                    label="invalid annotation",
                )

            annotation = args[0] if args[1] in (None, type(None)) else args[1]
            if isinstance(annotation, type) and issubclass(annotation, DataFrame):
                self._type_error(
                    (
                        f"Invalid type annotation for feature '{error_ctx}': "
                        f"'{annotation}' refers to an Optional DataFrame. "
                        "Since DataFrames can be empty, the Optional is unnecessary. "
                        "Please refer to your DataFrame as a non-optional type, e.g. `DataFrame[FeatureClassB]`."
                    ),
                    code="78",
                    label="invalid annotation",
                )
                return
            origin = get_origin(annotation)
            self._is_nullable = True

        if origin in (Annotated, getattr(typing, "Annotated", Annotated)):
            args = get_args(annotation)
            annotation = args[0]
            if "__chalk_ts__" in args:
                self._is_feature_time = True
            if "__chalk_primary__" in args:
                self._is_primary = True
            if "__chalk_document__" in args:
                self._is_document = True
            origin = get_origin(annotation)

        # The only allowed collections here are Set, List, or DataFrame
        if origin in (set, Set):
            args = get_args(annotation)
            if len(args) != 1:
                self._type_error(
                    f"Set takes exactly one arg, but found {len(args)} type parameters",
                    code="72",
                    label="invalid set",
                )
            annotation = args[0]
        if origin in (frozenset, FrozenSet):
            args = get_args(annotation)
            if len(args) != 1:
                self._type_error(
                    f"FrozenSet takes exactly one arg, but found {len(args)} type parameters",
                    code="73",
                    label="invalid frozen set",
                )
            annotation = args[0]
        if origin in (tuple, Tuple):
            args = get_args(annotation)
            if len(args) != 2 or args[1] is not ... or args[0] is ...:
                self._type_error(
                    (
                        "Tuple should be given exactly two type parameters. "
                        "The first should be the type of the elements, and the second should be '...', "
                        "which indicates that the tuple is of variable length. "
                        "For example, 'Tuple[int, ...]' is a tuple of ints of variable length."
                    ),
                    code="74",
                    label="invalid tuple",
                )
            annotation = args[0]
        if origin in (list, List):
            args = get_args(annotation)
            if len(args) != 1:
                self._type_error(
                    f"List takes exactly one arg, but found {len(args)} type parameters",
                    code="75",
                    label="invalid list",
                )
            annotation = args[0]

        if origin in (dict, Dict):
            args = get_args(annotation)
            if len(args) != 2:
                self._type_error(
                    f"Dict takes exactly two args, but found `{Dict[args]}`",  # pyright: ignore
                    code="156",
                    label="invalid dict",
                )
            if is_optional(args[0]):
                self._type_error(
                    f"Dict keys cannot be optional types, found `{Dict[args]}`",  # pyright: ignore
                    code="157",
                    label="optional dict key not allowed",
                )
            if args[1] in (Any, object) or (get_origin(args[1]) == Union and not is_optional(args[1])):
                self._type_error(
                    f"Only homogeneous dicts supported, found `{Dict[args]}`",  # pyright: ignore
                    code="158",
                    label="`Any` as dict value type not allowed",
                )

            annotation = dict

        if isinstance(annotation, FeatureWrapper):
            # We never want FeatureWrappers; if this is the case, then unwrap it to the underlying feature
            annotation = unwrap_feature(annotation)

        if not isinstance(annotation, (type, Feature)) and annotation not in (
            JSON,
            HttpResponse[str],
            HttpResponse[bytes],
        ):
            if isinstance(annotation, str):
                self._type_error(
                    (
                        f"Invalid type annotation for feature '{error_ctx}': "
                        f"{self._parsed_annotation} seems to be an incorrectly formatted forward reference. "
                        f"Forward references must be surrounded by quotes, e.g. '\"list[object]\"', "
                        f"not 'list[\"object\"]'. "
                    ),
                    code="76",
                    label="invalid reference",
                )

            elif origin in (set, Set, frozenset, FrozenSet, list, List, tuple, Tuple):
                origin = cast(type, origin)
                self._type_error(
                    (
                        f"Invalid type annotation for feature '{error_ctx}': "
                        f"{origin.__name__} must be of scalar types, "
                        f"not {self._parsed_annotation}"
                    ),
                    code="77",
                    label="invalid generic",
                )

            else:
                self._type_error(
                    (
                        f"Invalid type annotation for feature '{error_ctx}': "
                        f"'{self._parsed_annotation}' does not reference a Python type, Chalk feature, "
                        "or a type annotation."
                    ),
                    code="78",
                    label="invalid annotation",
                )
            return

        self._underlying = annotation  # pyright: ignore[reportAttributeAccessIssue]
        if isinstance(self._underlying, type):
            if ProtobufMessage is not None and issubclass(self._underlying, ProtobufMessage):
                self._as_proto = self._underlying

            elif issubclass(self._underlying, Features):
                self._as_features_cls = self._underlying

            elif issubclass(self._underlying, DataFrame):
                if len(self._underlying.columns) == 0:
                    self._type_error(
                        (
                            f"Invalid type annotation for feature '{error_ctx}': "
                            f"'{self._parsed_annotation}' does not reference a Chalk feature class. "
                            "Please add your feature class to the type, e.g. `DataFrame[FeatureClassB]`. "
                        ),
                        code="78",
                        label="invalid annotation",
                    )
                    return
                if self._features_cls is not None and self._underlying.namespace == self._features_cls.namespace:
                    self._type_error(
                        (
                            f"Invalid type annotation for feature '{error_ctx}': "
                            f"'{self._parsed_annotation}' is a DataFrame that refers to its own feature class. "
                            "This is unsupported."
                        ),
                        code="78",
                        label="invalid annotation",
                    )
                    return

                self._as_dataframe = self._underlying

            elif issubclass(self._underlying, Vector):
                self._as_vector = self._underlying

            elif issubclass(self._underlying, timedelta):
                self._type_error(
                    (f"Invalid type annotation for feature '{error_ctx}': timedelta feature types are not supported."),
                    code="78",
                    label="invalid annotation",
                )
                return

        elif isinstance(self._underlying, Feature):
            self._as_feature = self._underlying

    def as_proto(self) -> Optional[Type[ProtobufMessage]]:
        if self._parsed_annotation is None:
            self._parse_annotation()
        return self._as_proto

    def as_document(self) -> Optional[Type[Document]]:
        if self._parsed_annotation is None:
            self._parse_annotation()
        if not self._is_document:
            return None
        return cast("Type[Document]", self._underlying)

    @property
    def is_nullable(self) -> bool:
        """Whether the type annotation is nullable."""
        if self._parsed_annotation is None:
            self._parse_annotation()
        assert self._is_nullable is not None
        return self._is_nullable

    def as_features_cls(self) -> Optional[Type[Features]]:
        if self._parsed_annotation is None:
            self._parse_annotation()
        return self._as_features_cls

    def as_dataframe(self) -> Optional[Type[DataFrame]]:
        if self._parsed_annotation is None:
            self._parse_annotation()
        return self._as_dataframe

    def as_vector(self) -> Optional[Type[Vector]]:
        if self._parsed_annotation is None:
            self._parse_annotation()
        return self._as_vector

    def as_feature(self) -> Optional[Feature]:
        if self._parsed_annotation is None:
            self._parse_annotation()
        return self._as_feature

    def is_primary(self) -> bool:
        if self._parsed_annotation is None:
            self._parse_annotation()
        return self._is_primary

    def is_feature_time(self) -> bool:
        if self._parsed_annotation is None:
            self._parse_annotation()
        return self._is_feature_time



from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class TimeUnit(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    TIME_UNIT_UNSPECIFIED: _ClassVar[TimeUnit]
    TIME_UNIT_SECOND: _ClassVar[TimeUnit]
    TIME_UNIT_MILLISECOND: _ClassVar[TimeUnit]
    TIME_UNIT_MICROSECOND: _ClassVar[TimeUnit]
    TIME_UNIT_NANOSECOND: _ClassVar[TimeUnit]

class FeatherCompression(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    FEATHER_COMPRESSION_UNSPECIFIED: _ClassVar[FeatherCompression]
    FEATHER_COMPRESSION_UNCOMPRESSED: _ClassVar[FeatherCompression]
    FEATHER_COMPRESSION_LZ4: _ClassVar[FeatherCompression]
    FEATHER_COMPRESSION_ZSTD: _ClassVar[FeatherCompression]

TIME_UNIT_UNSPECIFIED: TimeUnit
TIME_UNIT_SECOND: TimeUnit
TIME_UNIT_MILLISECOND: TimeUnit
TIME_UNIT_MICROSECOND: TimeUnit
TIME_UNIT_NANOSECOND: TimeUnit
FEATHER_COMPRESSION_UNSPECIFIED: FeatherCompression
FEATHER_COMPRESSION_UNCOMPRESSED: FeatherCompression
FEATHER_COMPRESSION_LZ4: FeatherCompression
FEATHER_COMPRESSION_ZSTD: FeatherCompression

class Schema(_message.Message):
    __slots__ = ("columns", "metadata")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    COLUMNS_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    columns: _containers.RepeatedCompositeFieldContainer[Field]
    metadata: _containers.ScalarMap[str, str]
    def __init__(
        self,
        columns: _Optional[_Iterable[_Union[Field, _Mapping]]] = ...,
        metadata: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class Field(_message.Message):
    __slots__ = ("name", "arrow_type", "nullable", "children", "metadata")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    NAME_FIELD_NUMBER: _ClassVar[int]
    ARROW_TYPE_FIELD_NUMBER: _ClassVar[int]
    NULLABLE_FIELD_NUMBER: _ClassVar[int]
    CHILDREN_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    name: str
    arrow_type: ArrowType
    nullable: bool
    children: _containers.RepeatedCompositeFieldContainer[Field]
    metadata: _containers.ScalarMap[str, str]
    def __init__(
        self,
        name: _Optional[str] = ...,
        arrow_type: _Optional[_Union[ArrowType, _Mapping]] = ...,
        nullable: bool = ...,
        children: _Optional[_Iterable[_Union[Field, _Mapping]]] = ...,
        metadata: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class FixedSizeBinary(_message.Message):
    __slots__ = ("length",)
    LENGTH_FIELD_NUMBER: _ClassVar[int]
    length: int
    def __init__(self, length: _Optional[int] = ...) -> None: ...

class Timestamp(_message.Message):
    __slots__ = ("time_unit", "timezone")
    TIME_UNIT_FIELD_NUMBER: _ClassVar[int]
    TIMEZONE_FIELD_NUMBER: _ClassVar[int]
    time_unit: TimeUnit
    timezone: str
    def __init__(self, time_unit: _Optional[_Union[TimeUnit, str]] = ..., timezone: _Optional[str] = ...) -> None: ...

class Decimal(_message.Message):
    __slots__ = ("precision", "scale")
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    SCALE_FIELD_NUMBER: _ClassVar[int]
    precision: int
    scale: int
    def __init__(self, precision: _Optional[int] = ..., scale: _Optional[int] = ...) -> None: ...

class List(_message.Message):
    __slots__ = ("field_type",)
    FIELD_TYPE_FIELD_NUMBER: _ClassVar[int]
    field_type: Field
    def __init__(self, field_type: _Optional[_Union[Field, _Mapping]] = ...) -> None: ...

class FixedSizeList(_message.Message):
    __slots__ = ("field_type", "list_size")
    FIELD_TYPE_FIELD_NUMBER: _ClassVar[int]
    LIST_SIZE_FIELD_NUMBER: _ClassVar[int]
    field_type: Field
    list_size: int
    def __init__(
        self, field_type: _Optional[_Union[Field, _Mapping]] = ..., list_size: _Optional[int] = ...
    ) -> None: ...

class Struct(_message.Message):
    __slots__ = ("sub_field_types",)
    SUB_FIELD_TYPES_FIELD_NUMBER: _ClassVar[int]
    sub_field_types: _containers.RepeatedCompositeFieldContainer[Field]
    def __init__(self, sub_field_types: _Optional[_Iterable[_Union[Field, _Mapping]]] = ...) -> None: ...

class Map(_message.Message):
    __slots__ = ("key_field", "item_field", "keys_sorted")
    KEY_FIELD_FIELD_NUMBER: _ClassVar[int]
    ITEM_FIELD_FIELD_NUMBER: _ClassVar[int]
    KEYS_SORTED_FIELD_NUMBER: _ClassVar[int]
    key_field: Field
    item_field: Field
    keys_sorted: bool
    def __init__(
        self,
        key_field: _Optional[_Union[Field, _Mapping]] = ...,
        item_field: _Optional[_Union[Field, _Mapping]] = ...,
        keys_sorted: bool = ...,
    ) -> None: ...

class ScalarListValue(_message.Message):
    __slots__ = ("arrow_data", "schema")
    ARROW_DATA_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    arrow_data: bytes
    schema: Schema
    def __init__(
        self, arrow_data: _Optional[bytes] = ..., schema: _Optional[_Union[Schema, _Mapping]] = ...
    ) -> None: ...

class ScalarTime32Value(_message.Message):
    __slots__ = ("time32_second_value", "time32_millisecond_value")
    TIME32_SECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME32_MILLISECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    time32_second_value: int
    time32_millisecond_value: int
    def __init__(
        self, time32_second_value: _Optional[int] = ..., time32_millisecond_value: _Optional[int] = ...
    ) -> None: ...

class ScalarTime64Value(_message.Message):
    __slots__ = ("time64_microsecond_value", "time64_nanosecond_value")
    TIME64_MICROSECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME64_NANOSECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    time64_microsecond_value: int
    time64_nanosecond_value: int
    def __init__(
        self, time64_microsecond_value: _Optional[int] = ..., time64_nanosecond_value: _Optional[int] = ...
    ) -> None: ...

class ScalarTimestampValue(_message.Message):
    __slots__ = (
        "time_microsecond_value",
        "time_nanosecond_value",
        "time_second_value",
        "time_millisecond_value",
        "timezone",
    )
    TIME_MICROSECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME_NANOSECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME_SECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME_MILLISECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIMEZONE_FIELD_NUMBER: _ClassVar[int]
    time_microsecond_value: int
    time_nanosecond_value: int
    time_second_value: int
    time_millisecond_value: int
    timezone: str
    def __init__(
        self,
        time_microsecond_value: _Optional[int] = ...,
        time_nanosecond_value: _Optional[int] = ...,
        time_second_value: _Optional[int] = ...,
        time_millisecond_value: _Optional[int] = ...,
        timezone: _Optional[str] = ...,
    ) -> None: ...

class StructValue(_message.Message):
    __slots__ = ("field_values", "fields")
    FIELD_VALUES_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    field_values: _containers.RepeatedCompositeFieldContainer[ScalarValue]
    fields: _containers.RepeatedCompositeFieldContainer[Field]
    def __init__(
        self,
        field_values: _Optional[_Iterable[_Union[ScalarValue, _Mapping]]] = ...,
        fields: _Optional[_Iterable[_Union[Field, _Mapping]]] = ...,
    ) -> None: ...

class ScalarFixedSizeBinary(_message.Message):
    __slots__ = ("values", "length")
    VALUES_FIELD_NUMBER: _ClassVar[int]
    LENGTH_FIELD_NUMBER: _ClassVar[int]
    values: bytes
    length: int
    def __init__(self, values: _Optional[bytes] = ..., length: _Optional[int] = ...) -> None: ...

class DecimalValue(_message.Message):
    __slots__ = ("value", "precision", "scale")
    VALUE_FIELD_NUMBER: _ClassVar[int]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    SCALE_FIELD_NUMBER: _ClassVar[int]
    value: bytes
    precision: int
    scale: int
    def __init__(
        self, value: _Optional[bytes] = ..., precision: _Optional[int] = ..., scale: _Optional[int] = ...
    ) -> None: ...

class Extension(_message.Message):
    __slots__ = ("name", "storage_type")
    NAME_FIELD_NUMBER: _ClassVar[int]
    STORAGE_TYPE_FIELD_NUMBER: _ClassVar[int]
    name: str
    storage_type: ArrowType
    def __init__(
        self, name: _Optional[str] = ..., storage_type: _Optional[_Union[ArrowType, _Mapping]] = ...
    ) -> None: ...

class ExtensionValue(_message.Message):
    __slots__ = ("extension_type", "storage_value")
    EXTENSION_TYPE_FIELD_NUMBER: _ClassVar[int]
    STORAGE_VALUE_FIELD_NUMBER: _ClassVar[int]
    extension_type: Extension
    storage_value: ScalarValue
    def __init__(
        self,
        extension_type: _Optional[_Union[Extension, _Mapping]] = ...,
        storage_value: _Optional[_Union[ScalarValue, _Mapping]] = ...,
    ) -> None: ...

class ScalarValue(_message.Message):
    __slots__ = (
        "null_value",
        "bool_value",
        "float64_value",
        "int64_value",
        "large_utf8_value",
        "timestamp_value",
        "date_64_value",
        "struct_value",
        "large_list_value",
        "time64_value",
        "duration_second_value",
        "duration_millisecond_value",
        "duration_microsecond_value",
        "duration_nanosecond_value",
        "utf8_value",
        "int8_value",
        "int16_value",
        "int32_value",
        "uint8_value",
        "uint16_value",
        "uint32_value",
        "uint64_value",
        "float16_value",
        "float32_value",
        "date_32_value",
        "time32_value",
        "list_value",
        "fixed_size_list_value",
        "map_value",
        "binary_value",
        "large_binary_value",
        "fixed_size_binary_value",
        "decimal128_value",
        "decimal256_value",
        "extension_value",
    )
    NULL_VALUE_FIELD_NUMBER: _ClassVar[int]
    BOOL_VALUE_FIELD_NUMBER: _ClassVar[int]
    FLOAT64_VALUE_FIELD_NUMBER: _ClassVar[int]
    INT64_VALUE_FIELD_NUMBER: _ClassVar[int]
    LARGE_UTF8_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_VALUE_FIELD_NUMBER: _ClassVar[int]
    DATE_64_VALUE_FIELD_NUMBER: _ClassVar[int]
    STRUCT_VALUE_FIELD_NUMBER: _ClassVar[int]
    LARGE_LIST_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME64_VALUE_FIELD_NUMBER: _ClassVar[int]
    DURATION_SECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    DURATION_MILLISECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    DURATION_MICROSECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    DURATION_NANOSECOND_VALUE_FIELD_NUMBER: _ClassVar[int]
    UTF8_VALUE_FIELD_NUMBER: _ClassVar[int]
    INT8_VALUE_FIELD_NUMBER: _ClassVar[int]
    INT16_VALUE_FIELD_NUMBER: _ClassVar[int]
    INT32_VALUE_FIELD_NUMBER: _ClassVar[int]
    UINT8_VALUE_FIELD_NUMBER: _ClassVar[int]
    UINT16_VALUE_FIELD_NUMBER: _ClassVar[int]
    UINT32_VALUE_FIELD_NUMBER: _ClassVar[int]
    UINT64_VALUE_FIELD_NUMBER: _ClassVar[int]
    FLOAT16_VALUE_FIELD_NUMBER: _ClassVar[int]
    FLOAT32_VALUE_FIELD_NUMBER: _ClassVar[int]
    DATE_32_VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME32_VALUE_FIELD_NUMBER: _ClassVar[int]
    LIST_VALUE_FIELD_NUMBER: _ClassVar[int]
    FIXED_SIZE_LIST_VALUE_FIELD_NUMBER: _ClassVar[int]
    MAP_VALUE_FIELD_NUMBER: _ClassVar[int]
    BINARY_VALUE_FIELD_NUMBER: _ClassVar[int]
    LARGE_BINARY_VALUE_FIELD_NUMBER: _ClassVar[int]
    FIXED_SIZE_BINARY_VALUE_FIELD_NUMBER: _ClassVar[int]
    DECIMAL128_VALUE_FIELD_NUMBER: _ClassVar[int]
    DECIMAL256_VALUE_FIELD_NUMBER: _ClassVar[int]
    EXTENSION_VALUE_FIELD_NUMBER: _ClassVar[int]
    null_value: ArrowType
    bool_value: bool
    float64_value: float
    int64_value: int
    large_utf8_value: str
    timestamp_value: ScalarTimestampValue
    date_64_value: int
    struct_value: StructValue
    large_list_value: ScalarListValue
    time64_value: ScalarTime64Value
    duration_second_value: int
    duration_millisecond_value: int
    duration_microsecond_value: int
    duration_nanosecond_value: int
    utf8_value: str
    int8_value: int
    int16_value: int
    int32_value: int
    uint8_value: int
    uint16_value: int
    uint32_value: int
    uint64_value: int
    float16_value: float
    float32_value: float
    date_32_value: int
    time32_value: ScalarTime32Value
    list_value: ScalarListValue
    fixed_size_list_value: ScalarListValue
    map_value: ScalarListValue
    binary_value: bytes
    large_binary_value: bytes
    fixed_size_binary_value: ScalarFixedSizeBinary
    decimal128_value: DecimalValue
    decimal256_value: DecimalValue
    extension_value: ExtensionValue
    def __init__(
        self,
        null_value: _Optional[_Union[ArrowType, _Mapping]] = ...,
        bool_value: bool = ...,
        float64_value: _Optional[float] = ...,
        int64_value: _Optional[int] = ...,
        large_utf8_value: _Optional[str] = ...,
        timestamp_value: _Optional[_Union[ScalarTimestampValue, _Mapping]] = ...,
        date_64_value: _Optional[int] = ...,
        struct_value: _Optional[_Union[StructValue, _Mapping]] = ...,
        large_list_value: _Optional[_Union[ScalarListValue, _Mapping]] = ...,
        time64_value: _Optional[_Union[ScalarTime64Value, _Mapping]] = ...,
        duration_second_value: _Optional[int] = ...,
        duration_millisecond_value: _Optional[int] = ...,
        duration_microsecond_value: _Optional[int] = ...,
        duration_nanosecond_value: _Optional[int] = ...,
        utf8_value: _Optional[str] = ...,
        int8_value: _Optional[int] = ...,
        int16_value: _Optional[int] = ...,
        int32_value: _Optional[int] = ...,
        uint8_value: _Optional[int] = ...,
        uint16_value: _Optional[int] = ...,
        uint32_value: _Optional[int] = ...,
        uint64_value: _Optional[int] = ...,
        float16_value: _Optional[float] = ...,
        float32_value: _Optional[float] = ...,
        date_32_value: _Optional[int] = ...,
        time32_value: _Optional[_Union[ScalarTime32Value, _Mapping]] = ...,
        list_value: _Optional[_Union[ScalarListValue, _Mapping]] = ...,
        fixed_size_list_value: _Optional[_Union[ScalarListValue, _Mapping]] = ...,
        map_value: _Optional[_Union[ScalarListValue, _Mapping]] = ...,
        binary_value: _Optional[bytes] = ...,
        large_binary_value: _Optional[bytes] = ...,
        fixed_size_binary_value: _Optional[_Union[ScalarFixedSizeBinary, _Mapping]] = ...,
        decimal128_value: _Optional[_Union[DecimalValue, _Mapping]] = ...,
        decimal256_value: _Optional[_Union[DecimalValue, _Mapping]] = ...,
        extension_value: _Optional[_Union[ExtensionValue, _Mapping]] = ...,
    ) -> None: ...

class ArrowType(_message.Message):
    __slots__ = (
        "none",
        "bool",
        "float64",
        "int64",
        "large_utf8",
        "timestamp",
        "date64",
        "struct",
        "large_list",
        "time64",
        "duration",
        "utf8",
        "int8",
        "int16",
        "int32",
        "uint8",
        "uint16",
        "uint32",
        "uint64",
        "float16",
        "float32",
        "date32",
        "time32",
        "list",
        "fixed_size_list",
        "binary",
        "large_binary",
        "fixed_size_binary",
        "decimal_128",
        "decimal_256",
        "map",
        "extension",
    )
    NONE_FIELD_NUMBER: _ClassVar[int]
    BOOL_FIELD_NUMBER: _ClassVar[int]
    FLOAT64_FIELD_NUMBER: _ClassVar[int]
    INT64_FIELD_NUMBER: _ClassVar[int]
    LARGE_UTF8_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    DATE64_FIELD_NUMBER: _ClassVar[int]
    STRUCT_FIELD_NUMBER: _ClassVar[int]
    LARGE_LIST_FIELD_NUMBER: _ClassVar[int]
    TIME64_FIELD_NUMBER: _ClassVar[int]
    DURATION_FIELD_NUMBER: _ClassVar[int]
    UTF8_FIELD_NUMBER: _ClassVar[int]
    INT8_FIELD_NUMBER: _ClassVar[int]
    INT16_FIELD_NUMBER: _ClassVar[int]
    INT32_FIELD_NUMBER: _ClassVar[int]
    UINT8_FIELD_NUMBER: _ClassVar[int]
    UINT16_FIELD_NUMBER: _ClassVar[int]
    UINT32_FIELD_NUMBER: _ClassVar[int]
    UINT64_FIELD_NUMBER: _ClassVar[int]
    FLOAT16_FIELD_NUMBER: _ClassVar[int]
    FLOAT32_FIELD_NUMBER: _ClassVar[int]
    DATE32_FIELD_NUMBER: _ClassVar[int]
    TIME32_FIELD_NUMBER: _ClassVar[int]
    LIST_FIELD_NUMBER: _ClassVar[int]
    FIXED_SIZE_LIST_FIELD_NUMBER: _ClassVar[int]
    BINARY_FIELD_NUMBER: _ClassVar[int]
    LARGE_BINARY_FIELD_NUMBER: _ClassVar[int]
    FIXED_SIZE_BINARY_FIELD_NUMBER: _ClassVar[int]
    DECIMAL_128_FIELD_NUMBER: _ClassVar[int]
    DECIMAL_256_FIELD_NUMBER: _ClassVar[int]
    MAP_FIELD_NUMBER: _ClassVar[int]
    EXTENSION_FIELD_NUMBER: _ClassVar[int]
    none: EmptyMessage
    bool: EmptyMessage
    float64: EmptyMessage
    int64: EmptyMessage
    large_utf8: EmptyMessage
    timestamp: Timestamp
    date64: EmptyMessage
    struct: Struct
    large_list: List
    time64: TimeUnit
    duration: TimeUnit
    utf8: EmptyMessage
    int8: EmptyMessage
    int16: EmptyMessage
    int32: EmptyMessage
    uint8: EmptyMessage
    uint16: EmptyMessage
    uint32: EmptyMessage
    uint64: EmptyMessage
    float16: EmptyMessage
    float32: EmptyMessage
    date32: EmptyMessage
    time32: TimeUnit
    list: List
    fixed_size_list: FixedSizeList
    binary: EmptyMessage
    large_binary: EmptyMessage
    fixed_size_binary: int
    decimal_128: Decimal
    decimal_256: Decimal
    map: Map
    extension: Extension
    def __init__(
        self,
        none: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        bool: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        float64: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        int64: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        large_utf8: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        timestamp: _Optional[_Union[Timestamp, _Mapping]] = ...,
        date64: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        struct: _Optional[_Union[Struct, _Mapping]] = ...,
        large_list: _Optional[_Union[List, _Mapping]] = ...,
        time64: _Optional[_Union[TimeUnit, str]] = ...,
        duration: _Optional[_Union[TimeUnit, str]] = ...,
        utf8: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        int8: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        int16: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        int32: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        uint8: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        uint16: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        uint32: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        uint64: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        float16: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        float32: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        date32: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        time32: _Optional[_Union[TimeUnit, str]] = ...,
        list: _Optional[_Union[List, _Mapping]] = ...,
        fixed_size_list: _Optional[_Union[FixedSizeList, _Mapping]] = ...,
        binary: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        large_binary: _Optional[_Union[EmptyMessage, _Mapping]] = ...,
        fixed_size_binary: _Optional[int] = ...,
        decimal_128: _Optional[_Union[Decimal, _Mapping]] = ...,
        decimal_256: _Optional[_Union[Decimal, _Mapping]] = ...,
        map: _Optional[_Union[Map, _Mapping]] = ...,
        extension: _Optional[_Union[Extension, _Mapping]] = ...,
    ) -> None: ...

class EmptyMessage(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



from __future__ import annotations

from dataclasses import dataclass
from datetime import date
from enum import Enum
from typing import Literal, Optional, Union

from typing_extensions import TypeAlias


class DayOfWeek(int, Enum):
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7


@dataclass(frozen=True)
class HolidayDayOfWeek:
    """A holiday that occurs on a specific day of the week, like the first Monday of January (New Year's Day)"""

    name: str
    """The name of the holiday"""

    month: Literal[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
    """The month in which the holiday occurs, 1 to 12"""

    day_of_week: DayOfWeek
    """The day of the week on which the holiday occurs"""

    nth_week: Literal[-1, 1, 2, 3, 4, 5]
    """The week of the month in which the holiday occurs. -1 means the last week of the month"""

    start_date: Optional[date] = None
    """The day in which the holiday is observed"""


@dataclass(frozen=True)
class HolidayDayOfMonth:
    """A holiday that occurs on a specific day of the month, like the 25th of December (Christmas Day)"""

    name: str
    """The name of the holiday"""

    month: Literal[1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
    """The month in which the holiday occurs, 1 to 12"""

    day_of_month: int
    """The day of the month on which the holiday occurs, 1 to 31"""

    nearest_workday: bool
    """If True, the holiday is observed on the nearest workday if it falls on a weekend"""

    start_date: Optional[date] = None
    """The first year in which the holiday is observed"""


Holiday: TypeAlias = Union[HolidayDayOfWeek, HolidayDayOfMonth]

USMemorialDay = HolidayDayOfWeek(
    name="Memorial Day",
    month=5,
    day_of_week=DayOfWeek.MONDAY,
    nth_week=-1,
)

USLaborDay = HolidayDayOfWeek(
    name="Labor Day",
    month=9,
    day_of_week=DayOfWeek.MONDAY,
    nth_week=1,
)

USColumbusDay = HolidayDayOfWeek(
    name="Columbus Day",
    month=10,
    day_of_week=DayOfWeek.MONDAY,
    nth_week=2,
)

USThanksgivingDay = HolidayDayOfWeek(
    name="Thanksgiving Day",
    month=11,
    day_of_week=DayOfWeek.THURSDAY,
    nth_week=4,
)

USMartinLutherKingJr = HolidayDayOfWeek(
    name="Birthday of Martin Luther King, Jr.",
    month=1,
    start_date=date(1986, 1, 1),
    day_of_week=DayOfWeek.MONDAY,
    nth_week=3,
)

USPresidentsDay = HolidayDayOfWeek(
    name="Washington’s Birthday",
    month=2,
    day_of_week=DayOfWeek.MONDAY,
    nth_week=3,
)

NewYearsDay = HolidayDayOfMonth(
    name="New Year's Day",
    month=1,
    day_of_month=1,
    nearest_workday=True,
)

JuneteenthNationalIndependenceDay = HolidayDayOfMonth(
    name="Juneteenth National Independence Day",
    month=6,
    start_date=date(2021, 6, 18),
    day_of_month=19,
    nearest_workday=True,
)

IndependenceDay = HolidayDayOfMonth(
    name="Independence Day",
    month=7,
    day_of_month=4,
    nearest_workday=True,
)

VeteransDay = HolidayDayOfMonth(
    name="Veterans Day",
    month=11,
    day_of_month=11,
    nearest_workday=True,
)

ChristmasDay = HolidayDayOfMonth(
    name="Christmas Day",
    month=12,
    day_of_month=25,
    nearest_workday=True,
)

USFederalHolidays = [
    NewYearsDay,
    USMartinLutherKingJr,
    USPresidentsDay,
    USMemorialDay,
    JuneteenthNationalIndependenceDay,
    IndependenceDay,
    USLaborDay,
    USColumbusDay,
    VeteransDay,
    USThanksgivingDay,
    ChristmasDay,
]



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ErrorCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    ERROR_CODE_INTERNAL_SERVER_ERROR_UNSPECIFIED: _ClassVar[ErrorCode]
    ERROR_CODE_PARSE_FAILED: _ClassVar[ErrorCode]
    ERROR_CODE_RESOLVER_NOT_FOUND: _ClassVar[ErrorCode]
    ERROR_CODE_INVALID_QUERY: _ClassVar[ErrorCode]
    ERROR_CODE_VALIDATION_FAILED: _ClassVar[ErrorCode]
    ERROR_CODE_RESOLVER_FAILED: _ClassVar[ErrorCode]
    ERROR_CODE_RESOLVER_TIMED_OUT: _ClassVar[ErrorCode]
    ERROR_CODE_UPSTREAM_FAILED: _ClassVar[ErrorCode]
    ERROR_CODE_UNAUTHENTICATED: _ClassVar[ErrorCode]
    ERROR_CODE_UNAUTHORIZED: _ClassVar[ErrorCode]
    ERROR_CODE_CANCELLED: _ClassVar[ErrorCode]
    ERROR_CODE_DEADLINE_EXCEEDED: _ClassVar[ErrorCode]

class ErrorCodeCategory(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    ERROR_CODE_CATEGORY_NETWORK_UNSPECIFIED: _ClassVar[ErrorCodeCategory]
    ERROR_CODE_CATEGORY_REQUEST: _ClassVar[ErrorCodeCategory]
    ERROR_CODE_CATEGORY_FIELD: _ClassVar[ErrorCodeCategory]

ERROR_CODE_INTERNAL_SERVER_ERROR_UNSPECIFIED: ErrorCode
ERROR_CODE_PARSE_FAILED: ErrorCode
ERROR_CODE_RESOLVER_NOT_FOUND: ErrorCode
ERROR_CODE_INVALID_QUERY: ErrorCode
ERROR_CODE_VALIDATION_FAILED: ErrorCode
ERROR_CODE_RESOLVER_FAILED: ErrorCode
ERROR_CODE_RESOLVER_TIMED_OUT: ErrorCode
ERROR_CODE_UPSTREAM_FAILED: ErrorCode
ERROR_CODE_UNAUTHENTICATED: ErrorCode
ERROR_CODE_UNAUTHORIZED: ErrorCode
ERROR_CODE_CANCELLED: ErrorCode
ERROR_CODE_DEADLINE_EXCEEDED: ErrorCode
ERROR_CODE_CATEGORY_NETWORK_UNSPECIFIED: ErrorCodeCategory
ERROR_CODE_CATEGORY_REQUEST: ErrorCodeCategory
ERROR_CODE_CATEGORY_FIELD: ErrorCodeCategory

class ChalkException(_message.Message):
    __slots__ = ("kind", "message", "stacktrace", "internal_stacktrace")
    KIND_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    STACKTRACE_FIELD_NUMBER: _ClassVar[int]
    INTERNAL_STACKTRACE_FIELD_NUMBER: _ClassVar[int]
    kind: str
    message: str
    stacktrace: str
    internal_stacktrace: str
    def __init__(
        self,
        kind: _Optional[str] = ...,
        message: _Optional[str] = ...,
        stacktrace: _Optional[str] = ...,
        internal_stacktrace: _Optional[str] = ...,
    ) -> None: ...

class ChalkError(_message.Message):
    __slots__ = (
        "code",
        "category",
        "message",
        "display_primary_key",
        "display_primary_key_fqn",
        "exception",
        "feature",
        "resolver",
    )
    CODE_FIELD_NUMBER: _ClassVar[int]
    CATEGORY_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_PRIMARY_KEY_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_PRIMARY_KEY_FQN_FIELD_NUMBER: _ClassVar[int]
    EXCEPTION_FIELD_NUMBER: _ClassVar[int]
    FEATURE_FIELD_NUMBER: _ClassVar[int]
    RESOLVER_FIELD_NUMBER: _ClassVar[int]
    code: ErrorCode
    category: ErrorCodeCategory
    message: str
    display_primary_key: str
    display_primary_key_fqn: str
    exception: ChalkException
    feature: str
    resolver: str
    def __init__(
        self,
        code: _Optional[_Union[ErrorCode, str]] = ...,
        category: _Optional[_Union[ErrorCodeCategory, str]] = ...,
        message: _Optional[str] = ...,
        display_primary_key: _Optional[str] = ...,
        display_primary_key_fqn: _Optional[str] = ...,
        exception: _Optional[_Union[ChalkException, _Mapping]] = ...,
        feature: _Optional[str] = ...,
        resolver: _Optional[str] = ...,
    ) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.common.v1.feature_values_pb2 import (
    GetFeatureValuesChartRequest,
    GetFeatureValuesChartResponse,
    GetFeatureValuesTimeSeriesChartRequest,
    GetFeatureValuesTimeSeriesChartResponse,
    GetFeatureValuesTimeSeriesChartV2Request,
    GetFeatureValuesTimeSeriesChartV2Response,
)
from chalk._gen.chalk.common.v1.query_log_pb2 import (
    GetQueryLogEntriesRequest,
    GetQueryLogEntriesResponse,
)
from chalk._gen.chalk.common.v1.query_values_pb2 import (
    GetQueryValuesRequest,
    GetQueryValuesResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class OfflineStoreServiceStub:
    """This service exposes endpoints for dealing with the offline store. It should never depend on the python graph."""

    def __init__(self, channel: Channel) -> None: ...
    GetQueryLogEntries: UnaryUnaryMultiCallable[
        GetQueryLogEntriesRequest,
        GetQueryLogEntriesResponse,
    ]
    GetQueryValues: UnaryUnaryMultiCallable[
        GetQueryValuesRequest,
        GetQueryValuesResponse,
    ]
    GetFeatureValuesChart: UnaryUnaryMultiCallable[
        GetFeatureValuesChartRequest,
        GetFeatureValuesChartResponse,
    ]
    GetFeatureValuesTimeSeriesChart: UnaryUnaryMultiCallable[
        GetFeatureValuesTimeSeriesChartRequest,
        GetFeatureValuesTimeSeriesChartResponse,
    ]
    GetFeatureValuesTimeSeriesChartV2: UnaryUnaryMultiCallable[
        GetFeatureValuesTimeSeriesChartV2Request,
        GetFeatureValuesTimeSeriesChartV2Response,
    ]

class OfflineStoreServiceServicer(metaclass=ABCMeta):
    """This service exposes endpoints for dealing with the offline store. It should never depend on the python graph."""

    @abstractmethod
    def GetQueryLogEntries(
        self,
        request: GetQueryLogEntriesRequest,
        context: ServicerContext,
    ) -> GetQueryLogEntriesResponse: ...
    @abstractmethod
    def GetQueryValues(
        self,
        request: GetQueryValuesRequest,
        context: ServicerContext,
    ) -> GetQueryValuesResponse: ...
    @abstractmethod
    def GetFeatureValuesChart(
        self,
        request: GetFeatureValuesChartRequest,
        context: ServicerContext,
    ) -> GetFeatureValuesChartResponse: ...
    @abstractmethod
    def GetFeatureValuesTimeSeriesChart(
        self,
        request: GetFeatureValuesTimeSeriesChartRequest,
        context: ServicerContext,
    ) -> GetFeatureValuesTimeSeriesChartResponse: ...
    @abstractmethod
    def GetFeatureValuesTimeSeriesChartV2(
        self,
        request: GetFeatureValuesTimeSeriesChartV2Request,
        context: ServicerContext,
    ) -> GetFeatureValuesTimeSeriesChartV2Response: ...

def add_OfflineStoreServiceServicer_to_server(servicer: OfflineStoreServiceServicer, server: Server) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/common/v1/chalk_error.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n!chalk/common/v1/chalk_error.proto\x12\x0f\x63halk.common.v1"\x8f\x01\n\x0e\x43halkException\x12\x12\n\x04kind\x18\x01 \x01(\tR\x04kind\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x1e\n\nstacktrace\x18\x03 \x01(\tR\nstacktrace\x12/\n\x13internal_stacktrace\x18\x04 \x01(\tR\x12internalStacktrace"\xe6\x03\n\nChalkError\x12.\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x1a.chalk.common.v1.ErrorCodeR\x04\x63ode\x12>\n\x08\x63\x61tegory\x18\x02 \x01(\x0e\x32".chalk.common.v1.ErrorCodeCategoryR\x08\x63\x61tegory\x12\x18\n\x07message\x18\x03 \x01(\tR\x07message\x12\x33\n\x13\x64isplay_primary_key\x18\x65 \x01(\tH\x00R\x11\x64isplayPrimaryKey\x88\x01\x01\x12:\n\x17\x64isplay_primary_key_fqn\x18\x66 \x01(\tH\x01R\x14\x64isplayPrimaryKeyFqn\x88\x01\x01\x12\x42\n\texception\x18g \x01(\x0b\x32\x1f.chalk.common.v1.ChalkExceptionH\x02R\texception\x88\x01\x01\x12\x1d\n\x07\x66\x65\x61ture\x18h \x01(\tH\x03R\x07\x66\x65\x61ture\x88\x01\x01\x12\x1f\n\x08resolver\x18i \x01(\tH\x04R\x08resolver\x88\x01\x01\x42\x16\n\x14_display_primary_keyB\x1a\n\x18_display_primary_key_fqnB\x0c\n\n_exceptionB\n\n\x08_featureB\x0b\n\t_resolver*\x99\x03\n\tErrorCode\x12\x30\n,ERROR_CODE_INTERNAL_SERVER_ERROR_UNSPECIFIED\x10\x00\x12\x1b\n\x17\x45RROR_CODE_PARSE_FAILED\x10\x01\x12!\n\x1d\x45RROR_CODE_RESOLVER_NOT_FOUND\x10\x02\x12\x1c\n\x18\x45RROR_CODE_INVALID_QUERY\x10\x03\x12 \n\x1c\x45RROR_CODE_VALIDATION_FAILED\x10\x04\x12\x1e\n\x1a\x45RROR_CODE_RESOLVER_FAILED\x10\x05\x12!\n\x1d\x45RROR_CODE_RESOLVER_TIMED_OUT\x10\x06\x12\x1e\n\x1a\x45RROR_CODE_UPSTREAM_FAILED\x10\x07\x12\x1e\n\x1a\x45RROR_CODE_UNAUTHENTICATED\x10\x08\x12\x1b\n\x17\x45RROR_CODE_UNAUTHORIZED\x10\t\x12\x18\n\x14\x45RROR_CODE_CANCELLED\x10\n\x12 \n\x1c\x45RROR_CODE_DEADLINE_EXCEEDED\x10\x0b*\x80\x01\n\x11\x45rrorCodeCategory\x12+\n\'ERROR_CODE_CATEGORY_NETWORK_UNSPECIFIED\x10\x00\x12\x1f\n\x1b\x45RROR_CODE_CATEGORY_REQUEST\x10\x01\x12\x1d\n\x19\x45RROR_CODE_CATEGORY_FIELD\x10\x02\x42\x84\x01\n\x13\x63om.chalk.common.v1B\x0f\x43halkErrorProtoP\x01\xa2\x02\x03\x43\x43X\xaa\x02\x0f\x43halk.Common.V1\xca\x02\x0f\x43halk\\Common\\V1\xe2\x02\x1b\x43halk\\Common\\V1\\GPBMetadata\xea\x02\x11\x43halk::Common::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.common.v1.chalk_error_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.common.v1B\017ChalkErrorProtoP\001\242\002\003CCX\252\002\017Chalk.Common.V1\312\002\017Chalk\\Common\\V1\342\002\033Chalk\\Common\\V1\\GPBMetadata\352\002\021Chalk::Common::V1"
    _globals["_ERRORCODE"]._serialized_start = 690
    _globals["_ERRORCODE"]._serialized_end = 1099
    _globals["_ERRORCODECATEGORY"]._serialized_start = 1102
    _globals["_ERRORCODECATEGORY"]._serialized_end = 1230
    _globals["_CHALKEXCEPTION"]._serialized_start = 55
    _globals["_CHALKEXCEPTION"]._serialized_end = 198
    _globals["_CHALKERROR"]._serialized_start = 201
    _globals["_CHALKERROR"]._serialized_end = 687
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from os import PathLike
from typing import Any, Dict, Optional, Union, overload

from chalk.sql._internal.incremental import IncrementalSettings
from chalk.sql._internal.integrations.athena import AthenaSourceImpl
from chalk.sql._internal.integrations.bigquery import BigQuerySourceImpl
from chalk.sql._internal.integrations.clickhouse import ClickhouseSourceImpl
from chalk.sql._internal.integrations.cloudsql import CloudSQLSourceImpl
from chalk.sql._internal.integrations.databricks import DatabricksSourceImpl
from chalk.sql._internal.integrations.dynamodb import DynamoDBSourceImpl
from chalk.sql._internal.integrations.mysql import MySQLSourceImpl
from chalk.sql._internal.integrations.postgres import PostgreSQLSourceImpl
from chalk.sql._internal.integrations.redshift import RedshiftSourceImpl
from chalk.sql._internal.integrations.snowflake import SnowflakeSourceImpl
from chalk.sql._internal.integrations.spanner import SpannerSourceImpl
from chalk.sql._internal.integrations.sqlite import SQLiteSourceImpl
from chalk.sql._internal.integrations.trino import TrinoSourceImpl
from chalk.sql._internal.sql_file_resolver import make_sql_file_resolver
from chalk.sql._internal.sql_source_group import SQLSourceGroup
from chalk.sql.finalized_query import FinalizedChalkQuery
from chalk.sql.protocols import (
    BaseSQLSourceProtocol,
    ChalkQueryProtocol,
    SQLSourceWithTableIngestProtocol,
    StringChalkQueryProtocol,
    TableIngestProtocol,
)


@overload
def SnowflakeSource() -> BaseSQLSourceProtocol:
    """Connect to the only configured Snowflake database.

    If you have only one Snowflake connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = SnowflakeSource()
    """


@overload
def SnowflakeSource(*, name: str, engine_args: Optional[Dict[str, Any]] = ...) -> BaseSQLSourceProtocol:
    """Chalk's injects environment variables to support data integrations.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = SnowflakeSource(name="RISK")
    """
    ...


@overload
def SnowflakeSource(
    *,
    name: str | None = ...,
    account_identifier: str | None = ...,
    warehouse: str = ...,
    user: str | None = ...,
    password: str | None = ...,
    db: str | None = ...,
    schema: str | None = ...,
    role: str | None = ...,
    private_key_b64: str | None = ...,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this PostgresSQLSource is used within SQL File Resolvers.
    account_identifier
        Your Snowflake account identifier.
    warehouse
        Snowflake warehouse to use.
    user
        Username to connect to Snowflake.
    password
        The password to use.
    db
        Database to use.
    schema
        Snowflake schema in the database to use.
    role
        Snowflake role name to use.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> snowflake = SnowflakeSource(
    ...     db=os.getenv("SNOWSQL_DATABASE"),
    ...     schema=os.getenv("SNOWSQL_SCHEMA"),
    ...     role=os.getenv("SNOWSQL_ROLE"),
    ...     warehouse=os.getenv("SNOWSQL_WAREHOUSE"),
    ...     user=os.getenv("SNOWSQL_USER"),
    ...     password=os.getenv("SNOWSQL_PWD"),
    ...     account_identifier=os.getenv("SNOWSQL_ACCOUNT_IDENTIFIER")
    ... )
    """
    ...


def SnowflakeSource(
    *,
    name: Optional[str] = None,
    account_identifier: Optional[str] = None,
    warehouse: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    db: Optional[str] = None,
    schema: Optional[str] = None,
    role: Optional[str] = None,
    private_key_b64: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """Create a Snowflake data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return SnowflakeSourceImpl(
        name=name,
        account_identifier=account_identifier,
        warehouse=warehouse,
        user=user,
        password=password,
        db=db,
        schema=schema,
        role=role,
        private_key_b64=private_key_b64,
        engine_args=engine_args,
    )


@overload
def PostgreSQLSource() -> SQLSourceWithTableIngestProtocol:
    """Connect to the only configured PostgreSQL database.

    If you have only one PostgreSQL connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> pg = PostgreSQLSource()
    """
    ...


@overload
def PostgreSQLSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
    async_engine_args: Optional[Dict[str, Any]] = ...,
) -> SQLSourceWithTableIngestProtocol:
    """If you have only one PostgreSQL integration, there's no need to provide
    a distinguishing name.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.
    async_engine_args
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = PostgreSQLSource(name="RISK")
    """
    ...


@overload
def PostgreSQLSource(
    *,
    name: str | None = ...,
    host: str | None = ...,
    port: int | str | None = ...,
    db: str | None = ...,
    user: str | None = ...,
    password: str | None = ...,
    engine_args: dict[str, Any] | None = ...,
    async_engine_args: dict[str, Any] | None = ...,
) -> SQLSourceWithTableIngestProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this PostgresSQLSource is used within SQL File Resolvers.
    host
        Name of host to connect to.
    port
        The port number to connect to at the server host.
    db
        The database name.
    user
        PostgreSQL username to connect as.
    password
        The password to be used if the server demands password authentication.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.
    async_engine_args
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> pg = PostgreSQLSource(
    ...     host=os.getenv("PGHOST"),
    ...     port=os.getenv("PGPORT"),
    ...     db=os.getenv("PGDATABASE"),
    ...     user=os.getenv("PGUSER"),
    ...     password=os.getenv("PGPASSWORD"),
    ... )
    >>> from chalk.features import online
    >>> @online
    ... def resolver_fn() -> User.name:
    ...     return pg.query_string("select name from users where id = 4").one()
    """
    ...


def PostgreSQLSource(
    *,
    host: Optional[str] = None,
    port: Optional[Union[int, str]] = None,
    db: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    name: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> TableIngestProtocol:
    """Create a PostgreSQL data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return PostgreSQLSourceImpl(
        host,
        port,
        db,
        user,
        password,
        name,
        engine_args=engine_args,
        async_engine_args=async_engine_args,
    )


@overload
def MySQLSource() -> SQLSourceWithTableIngestProtocol:
    """If you have only one MySQL connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> mysql = MySQLSource()
    """
    ...


@overload
def MySQLSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
    async_engine_args: Optional[Dict[str, Any]] = ...,
) -> SQLSourceWithTableIngestProtocol:
    """If you have only one MySQL integration, there's no need to provide
    a distinguishing name.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = MySQLSource(name="RISK")
    """
    ...


@overload
def MySQLSource(
    *,
    name: str | None = ...,
    host: str,
    port: Union[int, str] = ...,
    db: str = ...,
    user: str = ...,
    password: str = ...,
    engine_args: Optional[Dict[str, Any]] = ...,
    async_engine_args: Optional[Dict[str, Any]] = ...,
) -> SQLSourceWithTableIngestProtocol:
    """
    You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    host
        Name of host to connect to.
    port
        The port number to connect to at the server host.
    db
        The database name.
    user
        MySQL username to connect as.
    password
        The password to be used if the server demands password authentication.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.
    async_engine_args:
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> mysql = MySQLSource(
    ...     host=os.getenv("PGHOST"),
    ...     port=os.getenv("PGPORT"),
    ...     db=os.getenv("PGDATABASE"),
    ...     user=os.getenv("PGUSER"),
    ...     password=os.getenv("PGPASSWORD"),
    ... )
    >>> from chalk.features import online
    >>> @online
    ... def resolver_fn() -> User.name:
    ...     return mysql.query_string("select name from users where id = 4").one()
    """
    ...


def MySQLSource(
    *,
    host: Optional[str] = None,
    port: Optional[Union[int, str]] = None,
    db: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    name: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> SQLSourceWithTableIngestProtocol:
    """Create a MySQL data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return MySQLSourceImpl(
        host,
        port,
        db,
        user,
        password,
        name,
        engine_args=engine_args,
        async_engine_args=async_engine_args,
    )


def SQLiteInMemorySource(
    name: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> SQLSourceWithTableIngestProtocol:
    """Testing SQL source.

    If you have only one SQLiteInMemorySource integration, there's no need to provide
    a distinguishing name.

    Parameters
    ----------
    name
        The name of the integration.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.
    async_engine_args
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = SQLiteInMemorySource(name="RISK")
    """
    return SQLiteSourceImpl(name=name, engine_args=engine_args, async_engine_args=async_engine_args)


def SQLiteFileSource(
    filename: Union[str, PathLike],
    name: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> SQLSourceWithTableIngestProtocol:
    """Create a SQLite source for a file.

    Parameters
    ----------
    filename
        The name of the file.
    name
        The name to use in testing
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.
    async_engine_args
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    SQLSourceWithTableIngestProtocol
        The SQL source for use in Chalk resolvers.
    """
    return SQLiteSourceImpl(
        filename=filename,
        name=name,
        engine_args=engine_args,
        async_engine_args=async_engine_args,
    )


@overload
def RedshiftSource() -> BaseSQLSourceProtocol:
    """If you have only one Redshift connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = RedshiftSource()
    """
    ...


@overload
def RedshiftSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """If you have only one Redshift integration, there's no need to provide
    a distinguishing name.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = RedshiftSource(name="RISK")
    """
    ...


@overload
def RedshiftSource(
    *,
    name: str | None = ...,
    host: str = ...,
    db: str = ...,
    user: str = ...,
    password: str = ...,
    port: str = ...,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        The name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    host
        Name of host to connect to.
    db
        The database name.
    user
        Redshify username to connect as.
    password
        The password for the Redshift database.
    port
        The port number to use.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> redshift = RedshiftSource(
    ...     host=os.getenv("REDSHIFT_HOST"),
    ...     db=os.getenv("REDSHIFT_DB"),
    ...     user=os.getenv("REDSHIFT_USER"),
    ...     password=os.getenv("REDSHIFT_PASSWORD"),
    ... )
    >>> from chalk.features import online
    >>> @online
    ... def resolver_fn() -> User.name:
    ...     return redshift.query_string("select name from users where id = 4").one()
    """
    ...


def RedshiftSource(
    *,
    host: Optional[str] = None,
    db: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    name: Optional[str] = None,
    port: Optional[Union[str, int]] = None,
    engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """Create a Redshift data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return RedshiftSourceImpl(host, db, user, password, name, port, engine_args=engine_args)


@overload
def SpannerSource() -> BaseSQLSourceProtocol:
    """If you have only one Spanner connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = SpannerSource()
    """
    ...


@overload
def SpannerSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """If you have only one Spanner integration, there's no need to provide
    a distinguishing name.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = SpannerSource(name="RISK")
    """
    ...


@overload
def SpannerSource(
    *,
    name: str | None = None,
    project: str | None = None,
    instance: str | None = None,
    database: str | None = None,
    credentials_base64: str | None = None,
    emulator_host: str | None = None,
    engine_args: Dict[str, Any] | None = None,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    project
        The name of the GCP project for the Spanner instance.
    instance
        The name of the Spanner instance.
    database
        The name of the database in the Spanner instance.
    credentials_base64
        The credentials to use to connect, encoded as a base64 string.
    emulator_host
        Location of Spanner emulator, if desired.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> source = SpannerSource(
    ...     project=os.getenv("SPANNER_PROJECT"),
    ...     instance=os.getenv("SPANNER_INSTANCE"),
    ...     database=os.getenv("SPANNER_DATABASE"),
    ...     credentials_base64=os.getenv("SPANNER_CREDENTIALS_BASE64"),
    ... )
    """
    ...


def SpannerSource(
    name: str | None = None,
    project: str | None = None,
    instance: str | None = None,
    database: str | None = None,
    credentials_base64: str | None = None,
    emulator_host: str | None = None,
    engine_args: Dict[str, Any] | None = None,
) -> BaseSQLSourceProtocol:
    """Create a Spanner data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return SpannerSourceImpl(
        name=name,
        project=project,
        instance=instance,
        database=database,
        credentials_base64=credentials_base64,
        emulator_host=emulator_host,
        engine_args=engine_args,
    )


@overload
def BigQuerySource() -> BaseSQLSourceProtocol:
    """If you have only one BigQuery connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = BigQuerySource()
    """
    ...


@overload
def BigQuerySource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """If you have only one BigQuery integration, there's no need to provide
    a distinguishing name.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = BigQuerySource(name="RISK")
    """
    ...


@overload
def BigQuerySource(
    *,
    name: str | None = ...,
    project: Optional[str] = ...,
    dataset: Optional[str] = ...,
    location: Optional[str] = ...,
    credentials_base64: Optional[str] = ...,
    credentials_path: Optional[str] = ...,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    project
        The name of the GCP project for the BigQuery instance.
    dataset
        The name of the BigQuery dataset.
    location
        The location of the BigQuery instance.
    credentials_base64
        The credentials to use to connect, encoded as a base64 string.
    credentials_path
        The path to the credentials file to use to connect.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> source = BigQuerySource(
    ...     project=os.getenv("BIGQUERY_PROJECT"),
    ...     dataset=os.getenv("BIGQUERY_DATASET"),
    ...     location=os.getenv("BIGQUERY_LOCATION"),
    ...     credentials_base64=os.getenv("BIGQUERY_CREDENTIALS_BASE64"),
    ... )
    """
    ...


def BigQuerySource(
    *,
    name: Optional[str] = None,
    project: Optional[str] = None,
    dataset: Optional[str] = None,
    location: Optional[str] = None,
    credentials_base64: Optional[str] = None,
    credentials_path: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """Create a BigQuery data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return BigQuerySourceImpl(
        name=name,
        project=project,
        dataset=dataset,
        location=location,
        credentials_base64=credentials_base64,
        credentials_path=credentials_path,
        engine_args=engine_args,
    )


@overload
def CloudSQLSource() -> BaseSQLSourceProtocol:
    """If you have only one CloudSQL connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = CloudSQLSource()
    """
    ...


@overload
def CloudSQLSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """If you have only one CloudSQL integration, there's no need to provide
    a distinguishing name.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.
    async_engine_args
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = CloudSQLSource(name="RISK")
    """
    ...


@overload
def CloudSQLSource(
    *,
    name: str | None = ...,
    instance_name: Optional[str] = ...,
    db: Optional[str] = ...,
    user: Optional[str] = ...,
    password: Optional[str] = ...,
    engine_args: Optional[Dict[str, Any]] = ...,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    instance_name
        The name of the Cloud SQL instance, as defined in your GCP console.
    db
        Database to use.
    user
        Username to use.
    password
        The password to use.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.
    async_engine_args
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> CloudSQLSource(
    ...     instance_name=os.getenv("CLOUDSQL_INSTANCE_NAME"),
    ...     db=os.getenv("CLOUDSQL_DB"),
    ...     user=os.getenv("CLOUDSQL_USER"),
    ...     password=os.getenv("CLOUDSQL_PASSWORD"),
    ... )
    """


def CloudSQLSource(
    *,
    name: Optional[str] = None,
    instance_name: Optional[str] = None,
    db: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """Create a CloudSQL data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return CloudSQLSourceImpl(
        name=name,
        instance_name=instance_name,
        db=db,
        user=user,
        password=password,
        engine_args=engine_args,
        async_engine_args=async_engine_args,
    )


@overload
def TrinoSource() -> BaseSQLSourceProtocol:
    """Connect to the only configured Trino database.

    If you have only one Trino connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = TrinoSource()
    """
    ...


@overload
def TrinoSource(*, name: str, engine_args: Optional[Dict[str, Any]] = ...) -> BaseSQLSourceProtocol:
    """Chalk's injects environment variables to support data integrations.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = TrinoSource(name="RISK")
    """
    ...


@overload
def TrinoSource(
    *,
    name: Optional[str] = None,
    host: Optional[str] = None,
    port: Optional[Union[int, str]] = None,
    catalog: Optional[str] = None,
    schema: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    host
        Your Trino host.
    port
        Port number to use.
    catalog
        Catalog to use.
    schema
        Schema to use.
    user
        Trino username.
    password
        Trino password.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> trino = TrinoSource(
    ...     host=os.getenv("TRINO_HOST"),
    ...     port=os.getenv("TRINO_PORT"),
    ...     catalog=os.getenv("TRINO_CATALOG"),
    ...     schema=os.getenv("TRINO_SCHEMA"),
    ...     user=os.getenv("TRINO_USER"),
    ...     password=os.getenv("TRINO_PASSWORD"),
    ... )
    """
    ...


def TrinoSource(
    *,
    name: Optional[str] = None,
    host: Optional[str] = None,
    port: Optional[Union[int, str]] = None,
    catalog: Optional[str] = None,
    schema: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """Create a Trino data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return TrinoSourceImpl(host, port, catalog, schema, user, password, name, engine_args=engine_args)


@overload
def DatabricksSource() -> BaseSQLSourceProtocol:
    """Connect to the only configured Databricks database.

    If you have only one Databricks connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = DatabricksSource()
    """


@overload
def DatabricksSource(*, name: str, engine_args: Optional[Dict[str, Any]] = ...) -> BaseSQLSourceProtocol:
    """Chalk's injects environment variables to support data integrations.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = DatabricksSource(name="RISK")
    """
    ...


@overload
def DatabricksSource(
    *,
    name: str | None = ...,
    host: str = ...,
    http_path: str = ...,
    access_token: str = ...,
    db: str = ...,
    port: str = ...,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    host
        Your Databricks host.
    http_path
        Databricks HTTP path to use.
    access_token
        Access token to connect to Databricks.
    db
        Database to use.
    port
        Port number to use.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> databricks = DatabricksSource(
    ...     host=os.getenv("DATABRICKS_HOST"),
    ...     http_path=os.getenv("DATABRICKS_HTTP_PATH"),
    ...     access_token=os.getenv("DATABRICKS_TOKEN"),
    ...     db=os.getenv("DATABRICKS_DATABASE"),
    ...     port=os.getenv("DATABRICKS_PORT"),
    ... )
    """
    ...


def DatabricksSource(
    *,
    name: Optional[str] = None,
    host: Optional[str] = None,
    http_path: Optional[str] = None,
    access_token: Optional[str] = None,
    db: Optional[str] = None,
    port: Optional[Union[str, int]] = None,
    engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """Create a Databricks data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return DatabricksSourceImpl(
        host=host,
        http_path=http_path,
        access_token=access_token,
        db=db,
        port=port,
        name=name,
        engine_args=engine_args,
    )


@overload
def DynamoDBSource() -> BaseSQLSourceProtocol:
    """If you have only one DynamoDB connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.
    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.
    Examples
    --------
    >>> source = DynamoDBSource()
    """
    ...


@overload
def DynamoDBSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """If you have only one DynamoDB integration, there's no need to provide
    a distinguishing name.
    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        The name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = DynamoDBSource(name="RISK")
    """
    ...


@overload
def DynamoDBSource(
    *,
    name: str | None = None,
    aws_client_id_override: str | None = None,
    aws_client_secret_override: str | None = None,
    aws_role_arn_override: str | None = None,
    aws_region_override: str | None = None,
    endpoint_override: str | None = None,
    engine_args: Dict[str, Any] | None = None,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        The name of the integration. Not required unless if this SQL Source
        is used within SQL File Resolvers.
    aws_client_id_override
        Optionally override the credentials using an AWS client id, must be
        provided with a client secret override.
    aws_client_secret_override
        Optionally override the credentials using an AWS client secret, must
        be provided with a client id override.
    aws_role_arn_override
        Optionally override the credentials using an AWS role ARN.
    aws_region_override
        The AWS region code to connect to, such as `"us-east-1"`.
    endpoint_override
        Override for the DynamoDB endpoint if hosted elsewhere.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = DynamoDBSource(name="RISK")
    """
    ...


def DynamoDBSource(
    name: str | None = None,
    aws_client_id_override: str | None = None,
    aws_client_secret_override: str | None = None,
    aws_role_arn_override: str | None = None,
    aws_region_override: str | None = None,
    endpoint_override: str | None = None,
    engine_args: Dict[str, Any] | None = None,
) -> BaseSQLSourceProtocol:
    """Create a DynamoDB data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details. DynamoDBSources can be queried via PartiQL SQL
    resolvers.

    You may override the ambient AWS credentials by providing
    either a client ID and secret, or a role ARN.
    """
    return DynamoDBSourceImpl(
        name=name,
        aws_client_id_override=aws_client_id_override,
        aws_client_secret_override=aws_client_secret_override,
        aws_role_arn_override=aws_role_arn_override,
        aws_region_override=aws_region_override,
        endpoint_override=endpoint_override,
        engine_args=engine_args,
    )


@overload
def AthenaSource() -> BaseSQLSourceProtocol:
    """If you have only one Athena connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.
    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.
    Examples
    --------
    >>> source = AthenaSource()
    """
    ...


@overload
def AthenaSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """If you have only one Athena integration, there's no need to provide
    a distinguishing name.
    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        The name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = AthenaSource(name="RISK")
    """
    ...


@overload
def AthenaSource(
    *,
    name: str | None = None,
    aws_region: str | None = None,
    aws_access_key_id: str | None = None,
    aws_access_key_secret: str | None = None,
    s3_staging_dir: str | None = None,
    catalog_name: str | None = None,
    schema_name: str | None = None,
    role_arn: str | None = None,
    work_group: str | None = None,
    engine_args: Dict[str, Any] | None = None,
) -> BaseSQLSourceProtocol:
    """You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        The name of the integration. Not required unless if this SQL Source
        is used within SQL File Resolvers.
    aws_access_key_id
        Optionally override the credentials using an AWS client id, must be
        provided with a client secret override.
    aws_access_key_secret
        Optionally override the credentials using an AWS client secret, must
        be provided with a client id override.
    role_arn
        Optionally override the credentials using an AWS role ARN.
    aws_region
        The AWS region code to connect to, such as `"us-east-1"`.
    s3_staging_dir
        The query result location to store query results within s3.
    work_group
        Optionally provide an Athena work group to query from.
    catalog_name
        The catalog name to query. Defaults to "default".
    schema_name
        The schema (database) name to query. Defaults to "awsdatacatalog".
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = AthenaSource(name="RISK")
    """
    ...


def AthenaSource(
    name: str | None = None,
    aws_region: str | None = None,
    aws_access_key_id: str | None = None,
    aws_access_key_secret: str | None = None,
    s3_staging_dir: str | None = None,
    catalog_name: str | None = None,
    schema_name: str | None = None,
    role_arn: str | None = None,
    work_group: str | None = None,
    engine_args: Dict[str, Any] | None = None,
) -> BaseSQLSourceProtocol:
    """Create an Amazon Athena data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details. DynamoDBSources can be queried via PartiQL SQL
    resolvers.

    You may override the ambient AWS credentials by providing
    either a client ID and secret, or a role ARN.
    """
    return AthenaSourceImpl(
        name=name,
        aws_region=aws_region,
        aws_access_key_id=aws_access_key_id,
        aws_access_key_secret=aws_access_key_secret,
        s3_staging_dir=s3_staging_dir,
        schema_name=schema_name,
        catalog_name=catalog_name,
        work_group=work_group,
        role_arn=role_arn,
        engine_args=engine_args,
    )


@overload
def ClickhouseSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """If you have only one Clickhouse connection that you'd like
    to add to Chalk, you do not need to specify any arguments
    to construct the source in your code.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = ClickhouseSource()
    """
    ...


@overload
def ClickhouseSource(
    *,
    name: str,
    engine_args: Optional[Dict[str, Any]] = ...,
    async_engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """If you have only one Clickhouse integration, there's no need to provide
    a distinguishing name.

    But what happens when you have two data sources of the same kind?
    When you create a new data source from your dashboard,
    you have an option to provide a name for the integration.
    You can then reference this name in the code directly.

    Parameters
    ----------
    name
        Name of the integration, as configured in your dashboard.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
        merged with any default arguments from the named integration.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> source = ClickhouseSource(name="RISK")
    """
    ...


@overload
def ClickhouseSource(
    *,
    name: str | None = ...,
    host: str,
    port: Union[int, str] = ...,
    db: str = ...,
    user: str = ...,
    password: str = ...,
    use_tls: Union[bool, str] = ...,
    engine_args: Optional[Dict[str, Any]] = ...,
    async_engine_args: Optional[Dict[str, Any]] = ...,
) -> BaseSQLSourceProtocol:
    """
    You can also configure the integration directly using environment
    variables on your local machine or from those added through the
    generic environment variable support (https://docs.chalk.ai/docs/env-vars).

    Parameters
    ----------
    name
        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
    host
        Name of host to connect to.
    port
        The port number to connect to at the server host.
    db
        The database name.
    user
        Clickhouse username to connect as.
    password
        The password to be used if the server demands password authentication.
    use_tls
        Whether to use tls protocol when communicating with the clickhouse engine, required for certain ports.
        See https://clickhouse.com/docs/guides/sre/network-ports for more details. Defaults to True.
    engine_args
        Additional arguments to use when constructing the SQLAlchemy engine.
    async_engine_args:
        Additional arguments to use when constructing an async SQLAlchemy engine.

    Returns
    -------
    BaseSQLSourceProtocol
        The SQL source for use in Chalk resolvers.

    Examples
    --------
    >>> import os
    >>> source = ClickhouseSource(
    ...     host=os.getenv("CLICKHOUSE_HOST"),
    ...     port=os.getenv("CLICKHOUSE_PORT"),
    ...     db=os.getenv("CLICKHOUSE_DATABASE"),
    ...     user=os.getenv("CLICKHOUSE_USER"),
    ...     password=os.getenv("CLICKHOUSE_PASSWORD"),
    ...     use_tls=os.getenv("CLICKHOUSE_USE_TLS"),
    ... )
    >>> from chalk.features import online
    >>> @online
    ... def resolver_fn() -> User.name:
    ...     return source.query_string("select name from users where id = 4").one()
    """
    ...


def ClickhouseSource(
    *,
    name: Optional[str] = None,
    host: Optional[str] = None,
    port: Optional[Union[int, str]] = None,
    db: Optional[str] = None,
    user: Optional[str] = None,
    password: Optional[str] = None,
    use_tls: Optional[Union[bool, str]] = None,
    engine_args: Optional[Dict[str, Any]] = None,
    async_engine_args: Optional[Dict[str, Any]] = None,
) -> BaseSQLSourceProtocol:
    """Create a Clickhouse data source. SQL-based data sources
    created without arguments assume a configuration in your
    Chalk Dashboard. Those created with the `name=` keyword
    argument will use the configuration for the integration
    with the given name. And finally, those created with
    explicit arguments will use those arguments to configure
    the data source. See the overloaded signatures for more
    details.
    """
    return ClickhouseSourceImpl(
        name=name,
        host=host,
        port=port,
        db=db,
        user=user,
        password=password,
        use_tls=use_tls,
        engine_args=engine_args,
        async_engine_args=async_engine_args,
    )


__all__ = (
    "AthenaSource",
    "BaseSQLSourceProtocol",
    "BigQuerySource",
    "ChalkQueryProtocol",
    "ClickhouseSource",
    "CloudSQLSource",
    "DatabricksSource",
    "DynamoDBSource",
    "FinalizedChalkQuery",
    "IncrementalSettings",
    "MySQLSource",
    "PostgreSQLSource",
    "RedshiftSource",
    "SQLSourceGroup",
    "SQLSourceWithTableIngestProtocol",
    "SQLiteFileSource",
    "SQLiteInMemorySource",
    "SnowflakeSource",
    "SpannerSource",
    "StringChalkQueryProtocol",
    "TableIngestProtocol",
    "TrinoSource",
    "make_sql_file_resolver",
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/kubernetes/v1/events.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n chalk/kubernetes/v1/events.proto\x12\x13\x63halk.kubernetes.v1"`\n\x1a\x43halkKubernetesEventSeries\x12\x14\n\x05\x63ount\x18\x01 \x01(\x05R\x05\x63ount\x12,\n\x12last_observed_time\x18\x02 \x01(\x03R\x10lastObservedTime"\xa7\x02\n\x14\x43halkKubernetesEvent\x12\x12\n\x04type\x18\x01 \x01(\tR\x04type\x12\x16\n\x06reason\x18\x02 \x01(\tR\x06reason\x12\x16\n\x06\x61\x63tion\x18\x03 \x01(\tR\x06\x61\x63tion\x12\x31\n\x14reporting_controller\x18\x04 \x01(\tR\x13reportingController\x12\x1d\n\nevent_time\x18\x05 \x01(\x03R\teventTime\x12\x17\n\x04note\x18\x06 \x01(\tH\x00R\x04note\x88\x01\x01\x12L\n\x06series\x18\x07 \x01(\x0b\x32/.chalk.kubernetes.v1.ChalkKubernetesEventSeriesH\x01R\x06series\x88\x01\x01\x42\x07\n\x05_noteB\t\n\x07_seriesB\xb0\x01\n\x17\x63om.chalk.kubernetes.v1B\x0b\x45ventsProtoP\x01Z\x1akubernetes/v1;kubernetesv1\xa2\x02\x03\x43KX\xaa\x02\x13\x43halk.Kubernetes.V1\xca\x02\x13\x43halk\\Kubernetes\\V1\xe2\x02\x1f\x43halk\\Kubernetes\\V1\\GPBMetadata\xea\x02\x15\x43halk::Kubernetes::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.kubernetes.v1.events_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\027com.chalk.kubernetes.v1B\013EventsProtoP\001Z\032kubernetes/v1;kubernetesv1\242\002\003CKX\252\002\023Chalk.Kubernetes.V1\312\002\023Chalk\\Kubernetes\\V1\342\002\037Chalk\\Kubernetes\\V1\\GPBMetadata\352\002\025Chalk::Kubernetes::V1"
    _globals["_CHALKKUBERNETESEVENTSERIES"]._serialized_start = 57
    _globals["_CHALKKUBERNETESEVENTSERIES"]._serialized_end = 153
    _globals["_CHALKKUBERNETESEVENT"]._serialized_start = 156
    _globals["_CHALKKUBERNETESEVENT"]._serialized_end = 451
# @@protoc_insertion_point(module_scope)
